# coding=utf-8
# Copyright (c) 2014, 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Provides tests for the shader_test module """

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import os
import textwrap
try:
    import mock
except ImportError:
    from unittest import mock

import pytest
import six

from framework import status
from framework.test import shader_test

# pylint: disable=invalid-name,no-self-use,protected-access


class _Setup(object):
    def __init__(self):
        self.__patchers = []
        self.__patchers.append(mock.patch.dict(
            'framework.test.base.OPTIONS.env',
            {'PIGLIT_PLATFORM': 'foo'}))

    def setup(self, _):
        for patcher in self.__patchers:
            patcher.start()

    def teardown(self, _):
        for patcher in self.__patchers:
            patcher.stop()


_setup = _Setup()
setup_module = _setup.setup
teardown_module = _setup.teardown


class TestConfigParsing(object):
    """Tests for ShaderRunner config parsing."""

    @pytest.mark.parametrize('gles,operator,expected', [
        # pylint: disable=bad-whitespace
        ('2.0', '>=', 'shader_runner_gles2'),
        ('2.0', '<=', 'shader_runner_gles2'),
        ('2.0', '=',  'shader_runner_gles2'),
        ('2.0', '>',  'shader_runner_gles3'),
        ('3.0', '>=', 'shader_runner_gles3'),
        ('3.0', '<=', 'shader_runner_gles2'),
        ('3.0', '=',  'shader_runner_gles3'),
        ('3.0', '>',  'shader_runner_gles3'),
        ('3.0', '<',  'shader_runner_gles2'),
        ('3.1', '>=', 'shader_runner_gles3'),
        ('3.1', '<=', 'shader_runner_gles2'),
        ('3.1', '=',  'shader_runner_gles3'),
        ('3.1', '>',  'shader_runner_gles3'),
        ('3.1', '<',  'shader_runner_gles2'),
        ('3.2', '>=', 'shader_runner_gles3'),
        ('3.2', '<=', 'shader_runner_gles2'),
        ('3.2', '=',  'shader_runner_gles3'),
        ('3.2', '<',  'shader_runner_gles2'),
    ])
    def test_bin(self, gles, operator, expected, tmpdir):
        """Test that the shader_runner parse picks the correct binary."""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL ES {} {}
            GLSL ES >= 1.00

            [next section]
            """.format(operator, gles)))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert os.path.basename(test.command[0]) == expected

    def test_gles3_bin(self, tmpdir):
        """test.shader_test.ShaderTest: Identifies GLES3 tests successfully."""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL ES >= 3.0
            GLSL ES >= 3.00 es
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert os.path.basename(test.command[0]) == "shader_runner_gles3"

    def test_skip_gl_required(self, tmpdir):
        """test.shader_test.ShaderTest: populates gl_requirements properly"""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL >= 3.0
            GL_ARB_ham_sandwhich
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert test.require_extensions == {'GL_ARB_ham_sandwhich'}

    def test_skip_gl_version(self, tmpdir):
        """test.shader_test.ShaderTest: finds gl_version."""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL >= 2.0
            GL_ARB_ham_sandwhich
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert test.require_version == 2.0

    def test_skip_gles_version(self, tmpdir):
        """test.shader_test.ShaderTest: finds gles_version."""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL ES >= 2.0
            GL_ARB_ham_sandwhich
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert test.require_version == 2.0

    def test_skip_glsl_version(self, tmpdir):
        """test.shader_test.ShaderTest: finds glsl_version."""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL >= 2.1
            GLSL >= 1.20
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert test.require_shader == 1.2

    def test_skip_glsl_es_version(self, tmpdir):
        """test.shader_test.ShaderTest: finds glsl_es_version."""
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL ES >= 2.0
            GLSL ES >= 1.00
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert test.require_shader == 1.0

    def test_ignore_directives(self, tmpdir):
        """There are some directives for shader_runner that are not interpreted
        by the python layer, they are only for the C layer. These should be
        ignored by the python layer.
        """
        p = tmpdir.join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL >= 3.3
            GLSL >= 1.50
            GL_MAX_VERTEX_OUTPUT_COMPONENTS
            GL_MAX_FRAGMENT_UNIFORM_COMPONENTS
            GL_MAX_VERTEX_UNIFORM_COMPONENTS
            GL_MAX_VARYING_COMPONENTS
            GL_ARB_foobar
            """))
        test = shader_test.ShaderTest.new(six.text_type(p))

        assert test.require_version == 3.3
        assert test.require_shader == 1.50
        assert test.require_extensions == {'GL_ARB_foobar'}


class TestCommand(object):
    """Tests for the command property."""

    @pytest.fixture(scope='class')
    def test_file(self, tmpdir_factory):
        p = tmpdir_factory.mktemp('shader-test-command').join('test.shader_test')
        p.write(textwrap.dedent("""\
            [require]
            GL ES >= 3.0
            GLSL ES >= 3.00 es
            """))
        return six.text_type(p)

    def test_getter_adds_auto_and_fbo(self, test_file):
        """test.shader_test.ShaderTest: -auto and -fbo is added to the command.
        """
        test = shader_test.ShaderTest.new(test_file)
        assert '-auto' in test.command
        assert '-fbo' in test.command

    def test_setter_doesnt_add_auto_and_fbo(self, test_file):
        """Don't add -fbo or -auto to self._command when using the setter."""
        test = shader_test.ShaderTest.new(test_file)
        test.command += ['-newarg']
        assert '-auto' not in test._command
        assert '-fbo' not in test._command


class TestMultiShaderTest(object):
    """Tests for the MultiShaderTest class."""

    class TestConstructor(object):
        """Tests for the constructor object."""

        @pytest.fixture
        def inst(self, tmpdir):
            """A fixture that creates an instance to test."""
            one = tmpdir.join('foo.shader_test')
            one.write(textwrap.dedent("""\
                [require]
                GLSL >= 3.0

                [vertex shader]"""))
            two = tmpdir.join('bar.shader_test')
            two.write(textwrap.dedent("""\
                [require]
                GLSL >= 4.0

                [vertex shader]"""))

            return shader_test.MultiShaderTest.new(
                [six.text_type(one), six.text_type(two)])

        def test_prog(self, inst):
            assert os.path.basename(inst.command[0]) == 'shader_runner'

        def test_filenames(self, inst):
            assert os.path.basename(inst.command[1]) == 'foo.shader_test'
            assert os.path.basename(inst.command[2]) == 'bar.shader_test'

        def test_extra(self, inst):
            assert inst.command[3] == '-auto'

    @pytest.fixture
    def inst(self, tmpdir):
        """A fixture that creates an instance to test."""
        one = tmpdir.join('foo.shader_test')
        one.write(textwrap.dedent("""\
            [require]
            GLSL >= 3.0

            [vertex shader]"""))
        two = tmpdir.join('bar.shader_test')
        two.write(textwrap.dedent("""\
            [require]
            GLSL >= 4.0
            GL_ARB_ham_sandwhich

            [vertex shader]"""))

        return shader_test.MultiShaderTest.new(
            [six.text_type(one), six.text_type(two)])

    def test_resume(self, inst):
        actual = inst._resume(1)  # pylint: disable=protected-access
        assert os.path.basename(actual[0]) == 'shader_runner'
        assert os.path.basename(actual[1]) == 'bar.shader_test'
        assert os.path.basename(actual[2]) == '-auto'

    def test_skips_set(self, inst):
        assert inst.skips[0].shader_version == 3.0
        assert inst.skips[1].shader_version == 4.0
        assert inst.skips[1].extensions == {'GL_ARB_ham_sandwhich'}

    def test_process_skips(self, inst):
        expected = {'bar': status.SKIP, 'foo': status.NOTRUN}
        with mock.patch.object(inst.skips[0].info.core, 'shader_version', 3.0):
            inst._process_skips()
        assert dict(inst.result.subtests) == expected



# coding=utf-8
# Copyright (c) 2018 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Piglit integration for KC CTS' GTF tests

By default this will run GLES2, GLES3 and GLES31 test cases. Those
desiring to run only a subset of them should consider using the -t or
-x options to include or exclude tests.

For example:
./piglit run gtf_gles -c foo -t ES3- would run only ES3 tests (note
the dash to exclude ES31 tests)

This integration requires some configuration in piglit.conf, or the
use of environment variables.

In piglit.conf one should set the following:
[gtf_gles]:bin -- Path to the glcts binary
[gtf_gles]:extra_args -- any extra arguments to be passed to cts
(optional)

Alternatively (or in addition, since environment variables have
precedence), one could set:
PIGLIT_GTF_GLES_BIN -- environment equivalent of [gtf_gles]:bin
PIGLIT_GTF_GLES_EXTRA_ARGS -- environment equivalent of
[gtf_gles]:extra_args

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import itertools

from framework.test import deqp

__all__ = ['profile']

_GTF_BIN = deqp.get_option('PIGLIT_GTF_GLES_BIN', ('gtf_gles', 'bin'),
                           required=True)

_EXTRA_ARGS = deqp.get_option('PIGLIT_GTF_GLES_EXTRA_ARGS', ('gtf_gles', 'extra_args'),
                              default='').split()


class DEQPGTFTest(deqp.DEQPBaseTest):
    deqp_bin = _GTF_BIN

    @property
    def extra_args(self):
        return super(DEQPGTFTest, self).extra_args + \
            [x for x in _EXTRA_ARGS if not x.startswith('--deqp-case')]


# Add all of the suites by default, users can use filters to remove them.
profile = deqp.make_profile(  # pylint: disable=invalid-name
    itertools.chain(
        deqp.iter_deqp_test_cases(
            deqp.gen_caselist_txt(_GTF_BIN, 'GTF-GLES2-cases.txt', _EXTRA_ARGS)),
        deqp.iter_deqp_test_cases(
            deqp.gen_caselist_txt(_GTF_BIN, 'GTF-GLES3-cases.txt', _EXTRA_ARGS)),
        deqp.iter_deqp_test_cases(
            deqp.gen_caselist_txt(_GTF_BIN, 'GTF-GLES31-cases.txt', _EXTRA_ARGS)),
    ),
    DEQPGTFTest)



# coding=utf-8
# Copyright (c) 2017 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Piglit integration for the now open sourced Khronos CTS tests being
developed at https://github.com/KhronosGroup/VK-GL-CTS

This will run GL45 test cases only.

This integration requires some configuration in piglit.conf, or the use of
environment variables.

In piglit.conf one should set the following:
[khr_gl45]:bin -- Path to the glcts binary
[khr_gl45]:extra_args -- any extra arguments to be passed to cts (optional)

Alternatively (or in addition, since environment variables have precedence),
one could set:
PIGLIT_KHR_GL_BIN -- environment equivalent of [khr_gl45]:bin
PIGLIT_KHR_GL_EXTRA_ARGS -- environment equivalent of [khr_gl45]:extra_args

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import itertools

from framework.test import deqp
from framework.options import OPTIONS

__all__ = ['profile']

_KHR_BIN = deqp.get_option('PIGLIT_KHR_GL_BIN', ('khr_gl45', 'bin'),
                           required=True)

_KHR_MUSTPASS = deqp.get_option('PIGLIT_KHRGL45_MUSTPASS',
                                 ('khr_gl45', 'mustpasslist'),
                                 required=OPTIONS.deqp_mustpass)

_EXTRA_ARGS = deqp.get_option('PIGLIT_KHR_GL_EXTRA_ARGS', ('khr_gl45', 'extra_args'),
                              default='').split()


class DEQPKHRTest(deqp.DEQPBaseTest):
    deqp_bin = _KHR_BIN

    @property
    def extra_args(self):
        return super(DEQPKHRTest, self).extra_args + \
            [x for x in _EXTRA_ARGS if not x.startswith('--deqp-case')]

profile = deqp.make_profile(  # pylint: disable=invalid-name
    deqp.select_source(_KHR_BIN, 'KHR-GL45-cases.txt', _KHR_MUSTPASS,
                       _EXTRA_ARGS),
    DEQPKHRTest)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""All OpenCL tests that come with piglit, using default settings."""

# Because these profiles put all of the names in the global scope and rely on
# __all__ to hide private names there will be a lot errors from pylint about
# invalid constant names, they're not really fixable, so just hide them.
# pylint: disable=invalid-name

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import os

from framework.profile import TestProfile
from framework.test.piglit_test import PiglitCLTest, CLProgramTester, ROOT_DIR
from framework import grouptools
from .py_modules.constants import TESTS_DIR, GENERATED_TESTS_DIR

__all__ = ['profile']

profile = TestProfile()

# Custom
with profile.test_list.group_manager(PiglitCLTest, 'custom') as g:
    g(['cl-custom-run-simple-kernel'], 'Run simple kernel')
    g(['cl-custom-flush-after-enqueue-kernel'], 'Flush after enqueue kernel')
    g(['cl-custom-r600-create-release-buffer-bug'],
      'r600 create release buffer bug')
    g(['cl-custom-buffer-flags'], 'Buffer flags')

with profile.test_list.group_manager(PiglitCLTest, 'api') as g:
    # Platform
    g(['cl-api-get-platform-ids'], 'clGetPlatformIDs')
    g(['cl-api-get-platform-info'], 'clGetPlatformInfo')
    g(['cl-api-get-extension-function-address-for-platform'],
        'clGetExtensionFunctionAddressForPlatform')

    # Device
    g(['cl-api-get-device-ids'], 'clGetDeviceIDs')
    g(['cl-api-get-device-info'], 'clGetDeviceInfo')

    # Context
    g(['cl-api-create-context'], 'clCreateContext')
    g(['cl-api-create-context-from-type'], 'clCreateContextFromType')
    g(['cl-api-get-context-info'], 'clGetContextInfo')
    g(['cl-api-retain_release-context'],
      'clRetainContext and clReleaseContext')

    # Command Queues
    g(['cl-api-create-command-queue'], 'clCreateCommandQueue')
    g(['cl-api-retain_release-command-queue'],
      'clRetainComandQueue and clReleaseCommandQueue')
    g(['cl-api-get-command-queue-info'], 'clGetCommandQueueInfo')

    # Memory objects
    g(['cl-api-create-buffer'], 'clCreateBuffer')
    g(['cl-api-create-image'], 'clCreateImage')
    g(['cl-api-create-sampler'], 'clCreateSampler')
    g(['cl-api-enqueue-copy-buffer'], 'clEnqueueCopyBuffer')
    g(['cl-api-enqueue-copy-buffer-rect'], 'clEnqueueCopyBufferRect')
    g(['cl-api-enqueue-read_write-buffer'],
      'clEnqueueReadBuffer and clEnqueueWriteBuffer')
    g(['cl-api-enqueue-fill-buffer'], 'clEnqueueFillBuffer')
    g(['cl-api-enqueue-fill-image'], 'clEnqueueFillImage')
    g(['cl-api-enqueue-migrate-mem-objects'], 'clEnqueueMigrateMemObjects')
    g(['cl-api-get-mem-object-info'], 'clGetMemObjectInfo')
    g(['cl-api-get-image-info'], 'clGetImageInfo')
    g(['cl-api-retain_release-mem-object'],
      'clRetainMemObject and clReleaseMemObject')

    g(['cl-api-create-program-with-binary'], 'clCreateProgramWithBinary')
    g(['cl-api-create-program-with-source'], 'clCreateProgramWithSource')
    g(['cl-api-build-program'], 'clBuildProgram')
    g(['cl-api-compile-program'], 'clCompileProgram')
    g(['cl-api-link-program'], 'clLinkProgram')
    g(['cl-api-get-program-info'], 'clGetProgramInfo')
    g(['cl-api-get-program-build-info'], 'clGetProgramBuildInfo')
    g(['cl-api-retain_release-program'],
      'clRetainProgram and clReleaseProgram')
    g(['cl-api-unload-compiler'], 'clUnloadCompiler')

    # Kernel
    g(['cl-api-create-kernel'], 'clCreateKernel')
    g(['cl-api-create-kernels-in-program'], 'clCreateKernelsInProgram')
    g(['cl-api-get-kernel-arg-info'], 'clGetKernelArgInfo')
    g(['cl-api-get-kernel-info'], 'clGetKernelInfo')
    g(['cl-api-get-kernel-work-group-info'], 'clGetKernelWorkGroupInfo')
    g(['cl-api-retain_release-kernel'], 'clRetainKernel and clReleaseKernel')
    g(['cl-api-set-kernel-arg'], 'clSetKernelArg')

    # Event
    g(['cl-api-get-event-info'], 'clGetEventInfo')
    g(['cl-api-retain_release-event'], 'clRetainEvent and clReleaseEvent')

with profile.test_list.group_manager(PiglitCLTest, 'program') as g:
    g(['cl-program-max-work-item-sizes'],
      'Run kernel with max work item sizes')
    g(['cl-program-bitcoin-phatk'], 'Bitcoin: phatk kernel')
    g(['cl-program-predefined-macros'], 'Check predefined preprocessor macros')

with profile.test_list.group_manager(PiglitCLTest, 'interop') as g:
    g(['cl-interop-egl_khr_cl_event2'], 'EGL_KHR_cl_event2')


def add_program_test_dir(group, dirpath, buildbase, installbase):
    for filename in os.listdir(os.path.join(buildbase, dirpath)):
        testname, ext = os.path.splitext(filename)
        if ext not in ['.cl', '.program_test']:
            continue

        profile.test_list[grouptools.join(group, testname)] = CLProgramTester(
            os.path.join(installbase, dirpath, os.path.basename(filename)))


base_test_dir = os.path.basename(TESTS_DIR)
base_gen_dir = os.path.basename(GENERATED_TESTS_DIR)

add_program_test_dir(grouptools.join('program', 'build'),
                     os.path.join('cl', 'program', 'build'),
                     TESTS_DIR,
                     base_test_dir)
add_program_test_dir(grouptools.join('program', 'build', 'fail'),
                     os.path.join('cl', 'program', 'build', 'fail'),
                     TESTS_DIR,
                     base_test_dir)
add_program_test_dir(grouptools.join('program', 'execute'),
                     os.path.join('cl', 'program', 'execute'),
                     TESTS_DIR,
                     base_test_dir)
add_program_test_dir(grouptools.join('program', 'execute'),
                     os.path.join('cl', 'program', 'execute', 'builtin', 'atomic'),
                     TESTS_DIR,
                     base_test_dir)
add_program_test_dir(grouptools.join('program', 'execute'),
                     os.path.join('cl', 'program', 'execute', 'builtin', 'convert'),
                     TESTS_DIR,
                     base_test_dir)

# Run generated built-in tests
add_program_test_dir(grouptools.join('program', 'execute', 'builtin'),
                     os.path.join('cl', 'builtin', 'int'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'builtin'),
                     os.path.join('cl', 'builtin', 'math'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'builtin'),
                     os.path.join('cl', 'builtin', 'relational'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'builtin'),
                     os.path.join('cl', 'builtin', 'common'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'builtin'),
                     os.path.join('cl', 'builtin', 'misc'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'store'),
                     os.path.join('cl', 'store'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'vstore'),
                     os.path.join('cl', 'vstore'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)
add_program_test_dir(grouptools.join('program', 'execute', 'vload'),
                     os.path.join('cl', 'vload'),
                     GENERATED_TESTS_DIR,
                     base_gen_dir)



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) 2016 Broadcom
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import argparse
import re
import sys
import os
import subprocess

def piglit_report_result(result):
    str = 'PIGLIT: {{"result": "{}" }}\n'.format(result)
    if result == 'warn' or result == 'fail':
        sys.stderr.write(str)
        exit(1)
    else:
        sys.stdout.write(str)
        exit(0)

def piglit_merge_result(result, sub_result):
    if result == 'skip':
        return sub_result
    elif sub_result == 'pass':
        return result
    elif sub_result == 'fail':
        return sub_result
    else:
        print('unknown sub result {}'.format(sub_result))
        piglit_report_result('fail')

def run_and_collect_output(filename):
    # Save off the path to the trace, minus '.trace'
    trace_prefix = os.path.splitext(filename)[0]

    # Place our generated .pngs in the directory of the trace, named
    # "tracename.out.drawcall.png".
    trace_png_prefix='{}.out.'.format(trace_prefix)

    command = [
        'apitrace',
        'replay',
        '-b', # No performance debugging or glGetError()s, please.
        '-v', # Get the "Wrote ..." lines.
        '-s', # Write pngs with the following prefix.
        trace_png_prefix,
        filename
    ]

    try:
        p = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        (stdout, stderr) = p.communicate()
        apitrace_output = (stdout + stderr).decode("utf-8")
    except KeyboardInterrupt:
        exit(1)
    except Exception:
        piglit_report_result('fail')

    return apitrace_output

def compare_images(out, expected):
    """Uses apitrace diff-images to compare two images, returning a

    piglit status string.
    """
    command = [
        'apitrace',
        'diff-images',
        '-a', # alpha channel counts
        out,
        expected,
    ]

    retcode = subprocess.call(command)
    if retcode == 0:
        return 'pass'
    elif retcode == 1:
        return 'fail'
    else:
        print('unknown apitrace diff-images return code {}'.format(retcode))
        piglit_report_result('fail')

def run_trace(filename, driver_categories):
    apitrace_output = run_and_collect_output(filename)

    result = 'skip'
    for line in apitrace_output.splitlines():
        m = re.match(r'Wrote (.*)\.out\.(.*).png', line)
        if m is None:
            continue

        prefix = m.group(1)
        drawcall = m.group(2)
        out_file = '{}.out.{}.png'.format(prefix, drawcall)

        # Go through the list of categories our driver
        for category in driver_categories:
            expected_file = '{}.expected.{}.{}.png'.format(prefix,
                                                           category,
                                                           drawcall)
            print('looking for {}'.format(expected_file))
            if not os.path.isfile(expected_file):
                continue

            sub_result = compare_images(out_file, expected_file)

            result = piglit_merge_result(result, sub_result)
            break

    piglit_report_result(result)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("filename",
                        metavar="<file.trace>",
                        help="Path to apitrace file")
    parser.add_argument("driver_categories",
                        metavar="<driver,categories>",
                        help="Driver categories to look for expected images in, in least-specific to most-specific order, separated by commas")
    args = parser.parse_args()

    run_trace(args.filename, args.driver_categories.split(','))

if __name__ == "__main__":
    main()



# coding=utf-8
#
# Copyright © 2016 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

"""Generate in/out fp64 tests."""

from __future__ import print_function, division, absolute_import
import argparse
import os
import itertools

from templates import template_dir
from modules import utils

TEMPLATES = template_dir(os.path.basename(os.path.splitext(__file__)[0]))

def get_dir_name(ver, test_type):
    """Returns the directory name to save tests given a GLSL version."""

    assert isinstance(ver, str)
    assert isinstance(test_type, str)
    if ver == '150':
        feature_dir = 'arb_gpu_shader_fp64'
    else:
        feature_dir = 'glsl-' + ver[0] + '.' + ver[1:]

    return os.path.join('spec', feature_dir, test_type,
                        'inout')


def generate_compilation_tests(type_name, shader, ver, names_only):
    """Generate in/out GLSL compilation tests."""

    assert isinstance(type_name, str)
    assert shader in ('vert', 'frag')
    assert isinstance(ver, str)
    assert isinstance(names_only, bool)

    filename = os.path.join(
        get_dir_name(ver, 'compiler'),
        '{0}-{1}put-{2}.{3}'.format('fs' if shader == 'frag' else 'vs',
                                    'out' if shader == 'frag' else 'in',
                                    type_name, shader))

    print(filename)

    if not names_only:
        with open(filename, 'w') as test_file:
            test_file.write(TEMPLATES.get_template(
                'template.{0}.mako'.format(shader)).render_unicode(
                    glsl_version='{}.{}'.format(ver[0], ver[1:]),
                    glsl_version_int=ver,
                    type_name=type_name,
                    extra_params=',0.0' if type_name in ['dvec2', 'dvec3'] else ''))


def generate_execution_tests(type_name, ver, names_only):
    """Generate in/out shader runner tests."""

    assert isinstance(type_name, str)
    assert isinstance(ver, str)
    assert isinstance(names_only, bool)

    filename = os.path.join(
        get_dir_name(ver, 'execution'),
        'vs-out-fs-in-{0}.shader_test'.format(type_name))

    print(filename)

    if not names_only:
        with open(filename, 'w') as test_file:
            test_file.write(TEMPLATES.get_template(
                'template.shader_test.mako').render_unicode(
                    glsl_version='{}.{}'.format(ver[0], ver[1:]),
                    glsl_version_int=ver,
                    type_name=type_name))


def all_compilation_tests(names_only):
    """Creates all the combinations for in/out compilation tests."""

    assert isinstance(names_only, bool)
    type_names = ['double', 'dvec2', 'dvec3', 'dvec4',
                  'dmat2', 'dmat2x3', 'dmat2x4',
                  'dmat3x2', 'dmat3', 'dmat3x4',
                  'dmat4x2', 'dmat4x3', 'dmat4']
    shaders = ['frag', 'vert']
    glsl_ver = ['150', '400']
    if not names_only:
        for ver in glsl_ver:
            utils.safe_makedirs(get_dir_name(ver, 'compiler'))

    for t_name, shader, ver in itertools.product(type_names, shaders, glsl_ver):
        yield t_name, shader, ver, names_only


def all_execution_tests(names_only):
    """Creates all the combinations for in/out shader runner tests."""

    assert isinstance(names_only, bool)
    type_names = ['double', 'dvec2', 'dvec3', 'dvec4']
    glsl_ver = ['150', '400']
    if not names_only:
        for ver in glsl_ver:
            utils.safe_makedirs(get_dir_name(ver, 'execution'))

    for t_name, ver in itertools.product(type_names, glsl_ver):
        yield t_name, ver, names_only


def main():
    """Main function."""

    parser = argparse.ArgumentParser(
        description="Generate in/out tests for fp64")
    parser.add_argument(
        '--names-only',
        dest='names_only',
        action='store_true',
        default=False,
        help="Don't output files, just generate a list of filenames to stdout")
    args = parser.parse_args()

    for test_args in all_compilation_tests(args.names_only):
        generate_compilation_tests(*test_args)

    for test_args in all_execution_tests(args.names_only):
        generate_execution_tests(*test_args)


if __name__ == '__main__':
    main()



# coding=utf-8
#
# Copyright (c) 2012 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# This permission notice shall be included in all copies or
# substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHOR(S) BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
# AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
# OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

"""Integration for running intel-gpu-tools with the piglit framework.

To use this either configure piglit.conf's [igt] section, or set IGT_TEST_ROOT
to the root of a built igt directory.

This will stop if you are not running as root, or if there are other users of
drm. Even if you have rendernode support enabled.

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import os
import re
import subprocess

from framework import grouptools, exceptions, core, options
from framework import dmesg
from framework.profile import TestProfile, Test

__all__ = ['profile']


def check_environment():
    """Check that the environment that piglit is running in is appropriate.

    IGT requires root, debugfs to be mounted, and to be the only drm client.

    """
    debugfs_path = "/sys/kernel/debug/dri"
    if os.getuid() != 0:
        raise exceptions.PiglitInternalError(
            "Test Environment check: not root!")
    if not os.path.isdir(debugfs_path):
        raise exceptions.PiglitInternalError(
            "Test Environment check: debugfs not mounted properly!")
    for subdir in os.listdir(debugfs_path):
        if not os.path.isdir(os.path.join(debugfs_path, subdir)):
            continue
        clients = open(os.path.join(debugfs_path, subdir, "clients"), 'r')
        lines = clients.readlines()
        if len(lines) > 2:
            raise exceptions.PiglitInternalError(
                "Test Environment check: other drm clients running!")


if 'IGT_TEST_ROOT' in os.environ:
    IGT_TEST_ROOT = os.environ['IGT_TEST_ROOT']
else:
    IGT_TEST_ROOT = os.path.join(
        core.PIGLIT_CONFIG.required_get('igt', 'path'), 'tests')

if not os.path.exists(IGT_TEST_ROOT):
    raise exceptions.PiglitFatalError(
        'IGT directory does not exist. Missing: {}'.format(IGT_TEST_ROOT))

# check for the test lists
if os.path.exists(os.path.join(IGT_TEST_ROOT, 'test-list.txt')):
    TEST_LISTS = ['test-list.txt']
elif (os.path.exists(os.path.join(IGT_TEST_ROOT, 'single-tests.txt')) and
      os.path.exists(os.path.join(IGT_TEST_ROOT, 'multi-tests.txt'))):
    TEST_LISTS = ['single-tests.txt', 'multi-tests.txt']
else:
    raise exceptions.PiglitFatalError("intel-gpu-tools test lists not found.")


class IGTTestProfile(TestProfile):
    """Test profile for intel-gpu-tools tests."""

    def setup(self):
        if options.OPTIONS.execute:
            try:
                check_environment()
            except exceptions.PiglitInternalError as e:
                raise exceptions.PiglitFatalError(str(e))


profile = IGTTestProfile()  # pylint: disable=invalid-name


class IGTTest(Test):
    """Test class for running libdrm."""
    def __init__(self, binary, arguments=None):
        if arguments is None:
            arguments = []
        super(IGTTest, self).__init__(
            [os.path.join(IGT_TEST_ROOT, binary)] + arguments)
        self.timeout = 600

    def interpret_result(self):
        super(IGTTest, self).interpret_result()

        if self.result.returncode == 0:
            if not self.result.err:
                self.result.result = 'pass'
            else:
                self.result.result = 'warn'
        elif self.result.returncode == 77:
            self.result.result = 'skip'
        elif self.result.returncode == 78:
            self.result.result = 'timeout'
        elif self.result.returncode == 139:
            self.result.result = 'crash'
        else:
            self.result.result = 'fail'


def list_tests(listname):
    """Parse igt test list and return them as a list."""
    with open(os.path.join(IGT_TEST_ROOT, listname), 'r') as f:
        lines = (line.rstrip() for line in f.readlines())

    found_header = False

    for line in lines:
        if found_header:
            return line.split(" ")

        if "TESTLIST" in line:
            found_header = True

    return []


def add_subtest_cases(test):
    """Get subtest instances."""
    try:
        out = subprocess.check_output(
            [os.path.join(IGT_TEST_ROOT, test), '--list-subtests'],
            env=os.environ.copy(),
            universal_newlines=True)
    except subprocess.CalledProcessError as e:
        # a return code of 79 indicates there are no subtests
        if e.returncode == 79:
            profile.test_list[grouptools.join('igt', test)] = IGTTest(test)
        elif e.returncode != 0:
            print("Error: Could not list subtests for " + test)
        else:
            raise

        # If we reach here there are no subtests.
        return

    for subtest in (s for s in out.splitlines() if s):
        profile.test_list[grouptools.join('igt', test, subtest)] = \
            IGTTest(test, ['--run-subtest', subtest])


def populate_profile():
    tests = []
    for test_list in TEST_LISTS:
        tests.extend(list_tests(test_list))

    for test in tests:
        add_subtest_cases(test)


populate_profile()
profile.options['dmesg'] = dmesg.get_dmesg(True)
profile.options['dmesg'].regex = re.compile(r"(\[drm:|drm_|intel_|i915_)")



#!/usr/bin/env python
# coding=utf-8
#
# Copyright © 2014 The Piglit Project
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

"""Test passing variables from the vertex shader to the tessellation control
shader.

For every varying type create one tests that passes a scalar of that type
and one test that passes a two element array.
Copy a uniform value to the varying in the vertex shader and compare the
varying to the same uniform in the tessellation control shader.
If the values are equal draw the screen green, red otherwise.

Draw for tessellated quads. Each should cover one quarter of the screen.

This script outputs, to stdout, the name of each file it generates.

"""

from __future__ import print_function, division, absolute_import
import os
import sys
import random
import textwrap

from six.moves import range


class Test(object):
    def __init__(self, type_name, array, name):
        """Creates a test.

        type_name -- varying type to test (e.g.: vec4, mat3x2, int, ...)
        array -- number of array elements to test, None for no array
        name -- name of the variable to test

        """
        self.var_name = name or 'var'

        if self.var_name == 'gl_Position':
            self.var_type = 'vec4'
            self.var_array = None
        elif self.var_name == 'gl_PointSize':
            self.var_type = 'float'
            self.var_array = None
        elif self.var_name == 'gl_ClipDistance':
            self.var_type = 'float'
            self.var_array = 8
        else:
            self.var_type = type_name
            self.var_array = array

        if self.built_in:
            self.interface_name = 'gl_PerVertex'
            self.interface_vs_instance = ''
            self.interface_tcs_instance = 'gl_in'
        else:
            self.interface_name = 'v2tc_interface'
            self.interface_vs_instance = ''
            self.interface_tcs_instance = 'v2tc'

        if self.var_array:
            self.var_type_full = self.var_type + '[{0}]'.format(self.var_array)
        else:
            self.var_type_full = self.var_type

    @property
    def built_in(self):
        return self.var_name.startswith('gl_')

    @property
    def vs_var_ref(self):
        return '.' + self.var_name if self.interface_vs_instance else self.var_name

    @property
    def tcs_var_ref(self):
        return '.' + self.var_name if self.interface_tcs_instance else self.var_name

    @property
    def uniform_string(self):
        """Returns string for loading uniform data by the shader_runner."""
        data = self.test_data()
        uniforms = ''
        if self.var_array:
            for i in range(12):
                for j in range(self.var_array):
                    uniforms += 'uniform {0} reference[{1}].v[{2}] {3}\n'.format(
                        self.var_type, i, j, data[i*j])
        else:
            for i in range(12):
                uniforms += 'uniform {0} reference[{1}].v {2}\n'.format(
                    self.var_type,
                    i,
                    data[i])

        #strip last newline
        return uniforms[:-1]

    def components(self):
        """Returns the number of scalar components of the used data type."""
        n = 1

        if self.var_type.startswith('mat'):
            if 'x' in self.var_type:
                n *= int(self.var_type[-1])
                n *= int(self.var_type[-3])
            else:
                n *= int(self.var_type[-1])
                n *= int(self.var_type[-1])
        elif 'vec' in self.var_type:
            n *= int(self.var_type[-1])

        return n

    def test_data(self):
        """Returns random but deterministic data as a list of strings.

        n strings are returned containing c random values, each.
        Where n is the number of vertices times the array length and
        c is the number of components in the tested scalar data type.
        """
        random.seed(17)

        if self.var_array:
            n = self.var_array * 12
        else:
            n = 12

        if self.var_type.startswith('i'):
            rand = lambda: random.randint(-0x80000000, 0x7fffffff)
        elif self.var_type.startswith('u'):
            rand = lambda: random.randint(0, 0xffffffff)
        else:
            rand = lambda: ((-1 + 2 * random.randint(0, 1)) *
                            random.randint(0, 2**23-1) *
                            2.0**(random.randint(-126, 127)))

        c = self.components()

        ret = []
        for _ in range(n):
            ret.append(" ".join(str(rand()) for _ in range(c)))

        return ret

    def filename(self):
        """Returns the file name (including path) for the test."""
        if self.built_in:
            name = self.var_name
        elif self.var_array:
            name = self.var_type + '_{0}'.format(self.var_array)
        else:
            name = self.var_type
        return os.path.join('spec',
                            'arb_tessellation_shader',
                            'execution',
                            'tcs-input',
                            'tcs-input-{0}.shader_test'.format(name))

    def generate(self):
        """Generates and writes the test to disc."""
        test = textwrap.dedent("""\
            # Test generated by:
            # {generator_command}
            # Test tessellation control shader inputs
            [require]
            GLSL >= 1.50
            GL_ARB_tessellation_shader

            [vertex shader]
            uniform struct S0 {{
                    {self.var_type_full} v;
            }} reference[12];

            out {self.interface_name} {{
                    {self.var_type_full} {self.var_name};
            }} {self.interface_vs_instance};

            void main()
            {{
                    {self.interface_vs_instance}{self.vs_var_ref} = reference[gl_VertexID].v;
            }}

            [tessellation control shader]
            #extension GL_ARB_tessellation_shader : require
            layout(vertices = 3) out;

            uniform struct S0 {{
                    {self.var_type_full} v;
            }} reference[12];

            in {self.interface_name} {{
                    {self.var_type_full} {self.var_name};
            }} {self.interface_tcs_instance}[];

            out int pass[];

            void main()
            {{
                    const int vertices_in = 3;
                    int local_pass = 1;
                    for (int i = 0; i < vertices_in; ++i) {{
                            int vertex_ID = gl_PrimitiveID * vertices_in + i;
                            if ({self.interface_tcs_instance}[i]{self.tcs_var_ref} != reference[vertex_ID].v)
                                    local_pass = 0;
                    }}
                    pass[gl_InvocationID] = local_pass;
                    gl_TessLevelOuter = float[4](1.0, 1.0, 1.0, 1.0);
                    gl_TessLevelInner = float[2](1.0, 1.0);
            }}

            [tessellation evaluation shader]
            #extension GL_ARB_tessellation_shader : require
            layout(quads) in;

            in int pass[];

            out vec4 vert_color;

            void main()
            {{
                    const vec4 red = vec4(1, 0, 0, 1);
                    const vec4 green = vec4(0, 1, 0, 1);
                    vec2[3] position = vec2[3](
                            vec2(float(gl_PrimitiveID / 2) - 1.0, float(gl_PrimitiveID % 2) - 1.0),
                            vec2(float(gl_PrimitiveID / 2) - 0.0, float(gl_PrimitiveID % 2) - 1.0),
                            vec2(float(gl_PrimitiveID / 2) - 1.0, float(gl_PrimitiveID % 2) - 0.0)
                    );
                    gl_Position = vec4(position[0]
                                + (position[1] - position[0]) * gl_TessCoord[0]
                                + (position[2] - position[0]) * gl_TessCoord[1], 0.0, 1.0);
                    vert_color = green;
                    if (pass[0] == 0 || pass[1] == 0 || pass[2] == 0) {{
                            vert_color = red;
                    }}
            }}

            [fragment shader]

            in vec4 vert_color;

            out vec4 frag_color;

            void main()
            {{
                    frag_color = vert_color;
            }}

            [test]
            {self.uniform_string}
            draw arrays GL_PATCHES 0 12
            relative probe rgb (0.25, 0.25) (0.0, 1.0, 0.0)
            relative probe rgb (0.75, 0.25) (0.0, 1.0, 0.0)
            relative probe rgb (0.25, 0.75) (0.0, 1.0, 0.0)
            relative probe rgb (0.75, 0.75) (0.0, 1.0, 0.0)
            """)

        test = test.format(self=self, generator_command=" ".join(sys.argv))

        filename = self.filename()
        dirname = os.path.dirname(filename)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        with open(filename, 'w') as f:
            f.write(test)


def all_tests():
    for type_name in ['float', 'vec2', 'vec3', 'vec4',
                      'mat2', 'mat3', 'mat4',
                      'mat2x3', 'mat2x4', 'mat3x2',
                      'mat3x4', 'mat4x2', 'mat4x3',
                      'int', 'ivec2', 'ivec3', 'ivec4',
                      'uint', 'uvec2', 'uvec3', 'uvec4']:
        for array in [None, 2]:
            yield Test(type_name=type_name, array=array, name=None)
    for var in ['gl_Position', 'gl_PointSize', 'gl_ClipDistance']:
        yield Test(type_name=None, array=None, name=var)


def main():
    for test in all_tests():
        test.generate()
        print(test.filename())


if __name__ == '__main__':
    main()



# coding=utf-8
# Copyright (c) 2014-2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a few simple helpers for working with mako templates."""

from __future__ import absolute_import
import os
import sys
import getpass
import tempfile

import mako
from mako.template import Template
from mako.lookup import TemplateLookup


# Based on a similar setup in framework/summary
MAKO_TEMP_DIR = os.path.join(tempfile.gettempdir(),
                             getpass.getuser(),
                             'piglit',
                             'python-{}'.format(sys.version.split()[0]),
                             'mako-{}'.format(mako.__version__),
                             'generators')

TEMPLATE_DIR = os.path.abspath(os.path.dirname(__file__))

# Future functions to be used in all templates
_FUTURES = ['division']


def template_file(generator, template):
    """Get a single template file from the templates directory.

    If the generator uses more than one template use template_dir instead.

    Arguments:
    generator -- the name of the generator, which coresponds to the subdir of
                 the templates folder.
    template -- the name of the template to get. If getting a single template
                it's probably template.*.mako, where * is shader_test,
                glsl_parser_test, vert, frag, geom, etc.

    """
    assert os.path.sep not in generator, \
        'generator should be a filename, not a path'
    assert not generator.endswith('.py'), \
        'generator should not have a file extension'

    return Template(filename=os.path.join(TEMPLATE_DIR, generator, template),
                    module_directory=os.path.join(MAKO_TEMP_DIR, generator),
                    future_imports=_FUTURES,
                    output_encoding='utf-8')


def template_dir(generator):
    """Get a TemplateLookup object for a generator.

    This is useful if a generator uses more than one template.

    Arguments:
    generator -- the name of the generator, which coresponds to the subdir of
                 the templates folder.

    """
    assert os.path.sep not in generator, \
        'generator should be a filename, not a path'
    assert not generator.endswith('.py'), \
        'generator should not have a file extension'

    return TemplateLookup(
        directories=[os.path.join(TEMPLATE_DIR, generator)],
        module_directory=os.path.join(MAKO_TEMP_DIR, generator),
        future_imports=_FUTURES,
        output_encoding='utf-8')



# encoding=utf-8
# Copyright © 2014, 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module with tests for grouptools."""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import pytest

from framework import grouptools

# pylint: disable=no-self-use


class TestJoin(object):
    """Tests for the join function."""

    def test_basic(self):
        """grouptools.join: works correctly."""
        assert grouptools.join('g1', 'g2') == \
            grouptools.SEPARATOR.join(['g1', 'g2'])

    def test_no_trailing_separator(self):
        """grouptools.join: doesn't add trailing separator with empty
        element.
        """
        test = grouptools.join('g1', 'g2', '')
        assert not test.endswith(grouptools.SEPARATOR)

    def test_empty(self):
        """grouptools.join: empty values are not joined"""
        assert grouptools.join('', 'spec') == 'spec'


class TestSplit(object):
    """tests for the split function."""

    def test_basic(self):
        assert grouptools.split(grouptools.join('g', 't')) == ['g', 't']

    def test_input_empty(self):
        """grouptools.split: an empty input returns []."""
        assert grouptools.split('') == []


class TestFromPath(object):
    """Tests for the from_path function."""

    def test_nt(self):
        """grouptools.from_path: converts \\ to separator in nt paths."""
        assert grouptools.from_path('foo\\bar') == grouptools.join('foo', 'bar')

    def test_posix(self):
        """grouptools.from_path: converts / to separator in posix paths."""
        # Since we already have tests for grouptools.join we can trust it to do
        # the right thing here. This also means that the test doesn't need to
        # be updated if the separator is changed.
        assert grouptools.from_path('foo/bar') == grouptools.join('foo', 'bar')

    def test_dot(self):
        """grouptools.from_path: should convert '.' into ''."""
        assert grouptools.from_path('.') == ''


class TestCommonprefix(object):
    """tests for the common prefix function."""

    def test_empty(self):
        """grouptools.commonprefix: handles an empty value"""
        actual = grouptools.commonprefix((grouptools.join('foo', 'bar'), ''))
        assert actual == ''

    def test_none(self):
        """grouptools.commonprefix: returns '' when no values are the same"""
        assert grouptools.commonprefix(['foo', 'bar']) == ''

    def test_basic(self):
        expected = grouptools.commonprefix([grouptools.join('g1', 'g2', '1'),
                                            grouptools.join('g1', 'g2', '2')])
        assert expected == grouptools.join('g1', 'g2')


class TestFormat(object):
    """tests for the format function."""

    def test_basic(self):
        """grouptools.format: replaces grouptools.SEPARATOR with '/'"""
        test_str = grouptools.SEPARATOR.join(['foo', 'bar', 'boink'])
        assert grouptools.format(test_str) == 'foo/bar/boink'


class TestTestname(object):
    """Tests for the testname function."""

    def test_basic(self):
        assert grouptools.testname(grouptools.join('g1', 'g2', 't1')) == 't1'


class TestGroupname(object):
    """Tests for the groupname function."""

    def test_basic(self):
        assert grouptools.groupname(grouptools.join('g1', 'g2', 't1')) == \
            grouptools.join('g1', 'g2')


class TestSplitname(object):
    """Tests for the splitname function."""

    def test_basic(self):
        assert grouptools.splitname(grouptools.join('g1', 'g2', 't1')) == \
            (grouptools.join('g1', 'g2'), 't1')



# coding=utf-8
# Copyright (c) 2014, 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Tests for the backend package """

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import pytest
import six

from framework import backends

# pylint: disable=no-self-use



# Helpers


@pytest.yield_fixture
def mock_backend(mocker):
    """Add an extra backend for testing."""
    mocker.patch.dict(
        backends.BACKENDS,
        {'test_backend': backends.register.Registry(
            extensions=['.test_backend'],
            backend=None,
            load=lambda x, _: x,
            meta=None,
        )})
    yield


# Tests


class TestGetBackend(object):
    """Tests for the get_backend function."""

    @pytest.mark.parametrize("name,expected", [
        ('json', backends.json.JSONBackend),
        ('junit', backends.junit.JUnitBackend),
    ])
    def test_basic(self, name, expected):
        """Test that ensures the expected input and output."""
        assert backends.get_backend(name) is expected

    def test_unknown_backend(self):
        """backends.get_backend: An error is raised with an unknown backend."""
        with pytest.raises(backends.BackendError):
            backends.get_backend('obviously fake backend')

    def test_notimplemented(self, mock_backend):  # pylint: disable=redefined-outer-name,unused-argument
        """backends.get_backend: An error is raised if a backend isn't
        implemented.
        """
        with pytest.raises(backends.BackendNotImplementedError):
            backends.get_backend('test_backend')


class TestLoad(object):
    """Tests for the load function."""

    def test_basic(self, mocker, tmpdir):  # pylint: disable=unused-argument
        """backends.load: works as expected.

        This is an interesting function to test, because it is just a wrapper
        that returns a TestrunResult object. So most of the testing should be
        happening in the tests for each backend.

        However, we can test this by injecting a fake backend, and ensuring
        that we get back what we expect. What we do is inject list(), which
        means that we should get back [file_path], instead of just file_path,
        like the legitimate backends return.
        """
        mocker.patch.dict(
            backends.BACKENDS,
            {'test_backend': backends.register.Registry(
                extensions=['.test_backend'],
                backend=None,
                load=lambda x, _: [x],
                meta=None,
            )})

        p = tmpdir.join('foo.test_backend')
        p.write('foo')
        test = backends.load(six.text_type(p))
        assert [six.text_type(p)] == test

    def test_unknown(self, tmpdir):
        p = tmpdir.join('foo.test_extension')
        p.write('foo')

        with pytest.raises(backends.BackendError):
            backends.load(six.text_type(p))

    def test_interupted(self, tmpdir, mock_backend):  # pylint: disable=unused-argument,redefined-outer-name
        """backends.load: works for resuming (no extension known)."""
        tmpdir.mkdir('tests')
        with tmpdir.join('tests', '0.test_backend').open('w') as f:
            f.write('foo')

        backends.load(six.text_type(tmpdir))

    def test_notimplemented(self, tmpdir, mocker):
        """backends.load(): An error is raised if a loader isn't properly
        implmented.
        """
        mocker.patch.dict(
            backends.BACKENDS,
            {'test_backend': backends.register.Registry(
                extensions=['.test_backend'],
                backend=None,
                load=None,
                meta=None,
            )})
        p = tmpdir.join('foo.test_backend')
        p.write('foo')

        with pytest.raises(backends.BackendNotImplementedError):
            backends.load(six.text_type(p))

    def test_trailing_dot(self, mocker):
        """framework.backends.load: handles the result name ending in '.'.

        Basically if this reaches a BackendNotImplementedError, then the '.'
        was handled correctly, otherwise if it's '.' then we should reach the
        BackendError, which is incorrect.
        """
        mocker.patch.dict(
            backends.BACKENDS,
            {'test_backend': backends.register.Registry(
                extensions=['.test_backend'],
                backend=None,
                load=None,
                meta=None,
            )})
        with pytest.raises(backends.BackendNotImplementedError):
            backends.load('foo.test_backend..gz')

    def test_old(self, tmpdir, mock_backend):  # pylint: disable=unused-argument,redefined-outer-name
        """backends.load: Ignores files ending in '.old'.

        If this raises a BackendError it means it didn't find a backend to use,
        thus it skipped the file ending in '.old'.
        """
        tmpdir.mkdir('test')
        p = tmpdir.join('test')
        with p.join('results.test_backend.old').open('w') as f:
            f.write('foo')

        with pytest.raises(backends.BackendError):
            backends.load(six.text_type(p))


class TestSetMeta(object):
    """Tests for the set_meta function."""

    def test_basic(self, mocker):
        """Basic argument/output test."""
        mocker.patch.dict(
            backends.BACKENDS,
            {'test_backend': backends.register.Registry(
                extensions=['.test_backend'],
                backend=None,
                load=None,
                meta=lambda x: x.append('bar'),
            )})

        test = []
        backends.set_meta('test_backend', test)
        assert test == ['bar']

    def test_no_backened(self):
        """backends.set_meta: raises an error if there is no meta function."""
        with pytest.raises(backends.BackendError):
            backends.set_meta('foo', {})

    def test_notimplemented(self, mock_backend):  # pylint: disable=redefined-outer-name,unused-argument
        """backends.load(): An error is raised if a set_meta isn't properly
        implmented.
        """
        with pytest.raises(backends.BackendNotImplementedError):
            backends.set_meta('test_backend', {})



# encoding=utf-8
# Copyright © 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Shared data for backend tests."""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from framework.options import OPTIONS


INITIAL_METADATA = {
    'name': 'name',
    'options': dict(OPTIONS),
    'info': {
        'system': {}
    }
}

# This is current JSON data, in raw form with only the minimum required
# changes. This does not contain piglit specifc objects, only strings, floats,
# ints, and Nones (instead of JSON's null)
JSON = {
    "results_version": 10,
    "time_elapsed": {
        "start": 1469638791.2351687,
        "__type__": "TimeAttribute",
        "end": 1469638791.4387212
    },
    "tests": {
        "spec@!opengl 1.0@gl-1.0-readpixsanity": {
            "dmesg": "",
            "traceback": None,
            "err": "piglit: error: waffle_display_connect failed due to "
                   "WAFFLE_ERROR_UNKNOWN: open drm file for gbm failed\n",
            "subtests": {
                "__type__": "Subtests"
            },
            "out": "",
            "exception": None,
            "command": "/home/user/source/piglit/bin/gl-1.0-readpixsanity "
                       "-auto -fbo",
            "time": {
                "start": 1469638791.2383287,
                "__type__": "TimeAttribute",
                "end": 1469638791.2439244
            },
            "pid": [11768],
            "__type__": "TestResult",
            "returncode": 1,
            "result": "fail",
            "environment": ("PIGLIT_SOURCE_DIR=\"/home/user/source/piglit\" "
                            " PIGLIT_PLATFORM=\"gbm\"")
        }
    },
    "options": {
        "dmesg": False,
        "concurrent": "some",
        "include_filter": [],
        "monitored": False,
        "execute": True,
        "valgrind": False,
        "profile": [
            "sanity"
        ],
        "log_level": "quiet",
        "env": {
            "PIGLIT_SOURCE_DIR": "/home/user/source/piglit",
            "PIGLIT_PLATFORM": "gbm"
        },
        "platform": "gbm",
        "sync": False,
        "exclude_tests": [],
        "exclude_filter": []
    },
    "name": "foo",
    "__type__": "TestrunResult",
    "info": {
        "system": {
            "lspci": "00:00.0 Host bridge...",
        },
    },
    "totals": {
        "spec": {
            '__type__': 'Totals',
            "warn": 0,
            "timeout": 0,
            "skip": 0,
            "crash": 0,
            "pass": 0,
            "fail": 1,
            "dmesg-warn": 0,
            "incomplete": 0,
            "notrun": 0,
            "dmesg-fail": 0
        },
        "": {
            '__type__': 'Totals',
            "warn": 0,
            "timeout": 0,
            "skip": 0,
            "crash": 0,
            "pass": 0,
            "fail": 1,
            "dmesg-warn": 0,
            "incomplete": 0,
            "notrun": 0,
            "dmesg-fail": 0
        },
        "spec@!opengl 1.0": {
            '__type__': 'Totals',
            "warn": 0,
            "timeout": 0,
            "skip": 0,
            "crash": 0,
            "pass": 0,
            "fail": 1,
            "dmesg-warn": 0,
            "incomplete": 0,
            "notrun": 0,
            "dmesg-fail": 0
        },
        "root": {
            '__type__': 'Totals',
            "warn": 0,
            "timeout": 0,
            "skip": 0,
            "crash": 0,
            "pass": 0,
            "fail": 1,
            "dmesg-warn": 0,
            "incomplete": 0,
            "notrun": 0,
            "dmesg-fail": 0
        }
    }
}



# encoding=utf-8
# Copyright © 2014, 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Tests for framework.status.

This module does not have the comprehensive tests for all of the various
combinations of comparisons between the various kinds of functions. Instead, it
just asserts that the regressions, fixes, etc lists are as expected.

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import itertools

import pytest
import six

from framework import status


# Statuses from worst to last. NotRun is intentionally not in this list and
# tested separately because of upcoming features for it
STATUSES = [
    status.PASS,
    status.WARN,
    status.DMESG_WARN,
    status.FAIL,
    status.DMESG_FAIL,
    status.TIMEOUT,
    status.CRASH,
    status.INCOMPLETE,
]

# all statuses except pass are problems
PROBLEMS = STATUSES[1:]

# Create lists of fixes and regressions programmatically based on the STATUSES
# list. This means less code, and easier expansion changes.
REGRESSIONS = list(itertools.combinations(STATUSES, 2)) + \
              list(itertools.combinations([status.SKIP] + PROBLEMS, 2))
FIXES = list(itertools.combinations(reversed(STATUSES), 2)) + \
        list(itertools.combinations(list(reversed(PROBLEMS)) + [status.SKIP], 2))

# The statuses that don't cause changes when transitioning from one another
NO_OPS = [status.SKIP, status.NOTRUN]


@pytest.mark.raises(exception=status.StatusException)
def test_bad_lookup():
    """status.status_lookup: An unexepcted value raises a StatusException"""
    status.status_lookup('foobar')


@pytest.mark.raises(exception=TypeError)
def test_status_eq_raises():
    """status.Status: eq comparison to uncomparable object results in TypeError"""
    status.PASS == dict()


@pytest.mark.raises(exception=TypeError)
def test_nochangestatus_eq_raises():
    """status.NoChangeStatus: eq comparison to uncomparable type results in TypeError"""
    status.NOTRUN == dict()


@pytest.mark.raises(exception=TypeError)
def test_nochangestatus_ne_raises():
    """status.NoChangeStatus: ne comparison to uncomparable type results in TypeError"""
    status.NOTRUN != dict()


def test_status_in():
    """status.Status: A status can be looked up with 'x in y' synatx"""
    stat = status.PASS
    slist = ['pass']

    assert stat in slist


@pytest.mark.parametrize(
    'stat', itertools.chain(STATUSES, [status.SKIP, status.NOTRUN]))
def test_lookup(stat):
    status.status_lookup(stat)


@pytest.mark.parametrize('new,old', REGRESSIONS)
def test_regression(new, old):
    assert status.status_lookup(new) < status.status_lookup(old)


@pytest.mark.parametrize('new,old', FIXES)
def test_fixes(new, old):
    assert status.status_lookup(new) > status.status_lookup(old)


@pytest.mark.parametrize('new,old', itertools.permutations(STATUSES, 2))
def test_changes(new, old):
    assert status.status_lookup(new) != status.status_lookup(old)


@pytest.mark.parametrize('new,old', itertools.permutations(NO_OPS, 2))
def test_no_change(new, old):
    new = status.status_lookup(new)
    old = status.status_lookup(old)
    assert not new < old
    assert not new > old


@pytest.mark.parametrize("stat,op,expected", [
    (status.Status('Test', 0, (0, 0)), six.text_type, 'Test'),
    (status.Status('Test', 0, (0, 0)), six.binary_type, b'Test'),
    (status.Status('Test', 0, (0, 0)), int, 0),
    (status.Status('Test', 0, (0, 0)), repr, 'Status("Test", 0, (0, 0))'),
    (status.Status('Test', 0, (0, 0)), hash, hash('Test')),
    (status.NoChangeStatus('No'), hash, hash('No')),
])
def test_status_comparisons(stat, op, expected):
    """Test status.Status equality protocol."""
    assert op(stat) == expected



# coding=utf-8
#
# Copyright © 2014-2016 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

from __future__ import print_function, division, absolute_import

import os.path
from textwrap import dedent

from mako.template import Template

from modules import utils


def product(ps, *qss):
    """
    Generate the cartesian product of a number of lists of dictionaries.

    Generate a sequence with every possible combination of elements of
    the specified lists of dictionaries.  Each element of the result
    will be the union of some combination of dictionaries from the
    lists given as argument.  The 'name' attribute of each dictionary
    given as input is concatenated with the names of other
    dictionaries part of the same combination to give the 'name'
    attribute of the result.
    """
    for q in (product(*qss) if qss else [{}]):
        for p in ps:
            r = dict(p, **q)
            r['name'] = ''.join(s['name'] for s in (p, q) if s.get('name'))
            yield r


def gen(src, tests):
    """
    Expand a source template for the provided list of test definitions.

    Generate a test script for each element of the 'tests' iterable,
    each of them should be a dictionary of definitions that will be
    used as environment to render the source template.

    The 'path' attribute of each dictionary gives the filename each
    test will be written to.
    """
    template = Template(dedent(src))

    for t in tests:
        print(t['path'])
        utils.safe_makedirs(os.path.dirname(t['path']))
        with open(t['path'], 'w') as f:
            f.write(template.render(**t))


def gen_execution(src, tests):
    """
    Generate a shader runner test for each element of the 'tests'
    iterable.
    """
    return gen(src,
               (dict(t, path = os.path.join(
                   'spec', t['extension'], 'execution', t['api'],
                   t['name'] + '.shader_test')) for t in tests))


def gen_compiler(src, tests):
    """
    Generate a GLSL parser test for each element of the 'tests'
    iterable.
    """
    return gen(src,
               (dict(t, path = os.path.join(
                   'spec', t['extension'], 'compiler', t['api'],
                   t['name'] + '.' + t['shader_stage'])) for t in tests))


#
# Common test definitions independent of the framebuffer fetch
# extension.
#
common_defs = [{# Allocate and bind a framebuffer object of the given
                # format and number of samples.
                'bind_fb': lambda fmt, samples = 0:
                   'fb ms {0} 250 250 {1}'.format(fmt, samples) if samples > 0 else
                   ('texture storage 0 2D {0} (1 250 250)\n'
                    'fb tex 2d 0').format(fmt) if fmt[-1] == 'I' else
                   ('texture rgbw 0 (250, 250) {0}\n'
                    'fb tex 2d 0').format(fmt),

                # Resolve the contents of a framebuffer previously
                # allocated with 'bind_fb' of the given format and
                # samples into a newly allocated single-sample
                # framebuffer.
                'resolve_fb': lambda fmt, samples:
                   '' if samples == 0 else
                   ('texture storage 0 2D {0} (1 250 250)\n'
                    'fb draw tex 2d 0\n'
                    'blit color nearest\n'
                    'fb read tex 2d 0\n').format(fmt) if fmt[-1] == 'I' else
                   ('texture rgbw 0 (250, 250) {0}\n'
                    'fb draw tex 2d 0\n'
                    'blit color linear\n'
                    'fb read tex 2d 0').format(fmt),

                # Blit the contents of the current read framebuffer
                # into the window system framebuffer. XXX - Requires
                # GL(ES) 3.0+ since glBlitFramebuffer doesn't exist on
                # earlier API versions.
                'display_fb': lambda api_version:
                   '' if api_version < 3.0 else
                   ('fb draw winsys\n'
                    'blit color linear'),

                # Declare a fragment color array set up for
                # framebuffer fetch with the specified number of
                # elements (n=0 indicates a single scalar output).
                #
                # The fragment color array can be accessed using the
                # 'frag_data' and 'last_frag_data' macros defined
                # below to make sure the generated test is valid
                # irrespective of the GLSL version.
                'decl_frag_data': lambda api_version, layout, n = 0, \
                                         t = 'vec4', p = 'mediump':
                   '' if api_version < 3.0 and not layout and t == 'vec4' \
                                           and p == 'mediump' else
                   layout + ' ' + p + ' ' + t + ' gl_LastFragData[gl_MaxDrawBuffers];' \
                                                       if api_version < 3.0 else
                   layout + ' inout ' + p + ' ' + t + ' fcolor;' if n == 0 else
                   layout + ' inout ' + p + ' ' + t + ' fcolor[{0}];'.format(n),

                'frag_data': lambda api_version, i = -1:
                   'gl_FragData[{0}]'.format(max(0, i)) if api_version < 3.0 else
                   'fcolor[{0}]'.format(i) if i >= 0 else
                   'fcolor',

                'last_frag_data': lambda api_version, i = -1:
                   'gl_LastFragData[{0}]'.format(max(0, i)) if api_version < 3.0 else
                   'fcolor[{0}]'.format(i) if i >= 0 else
                   'fcolor'}]


#
# Common test definitions for all supported extensions.
#
all_defs = list(product(common_defs,
                       [{'extension': 'EXT_shader_framebuffer_fetch',
                         'layout': '',
                         'barrier': ''},
                        {'extension': 'EXT_shader_framebuffer_fetch_non_coherent',
                         'layout': 'layout(noncoherent)',
                         'barrier': 'fbfetch barrier'}]))


def main():
    """Main function."""

    #
    # Test that the GL(ES) 2 gl_LastFragData built-in is not defined
    # in more recent GLSL versions.
    #
    gen_compiler("""\
        /*
         * [config]
         * expect_result: fail
         * glsl_version: 3.0 es
         * require_extensions: GL_${extension}
         * [end config]
         */
        #version 300 es
        #extension GL_${extension} : enable

        out highp vec4 color;

        void main()
        {
            color = gl_LastFragData[0];
        }
    """, product(all_defs,
                 [{'name': 'negative-gl_LastFragData',
                   'api': 'gles3',
                   'shader_stage': 'frag'}]))

    #
    # Test that the GL(ES) 2 gl_LastFragData built-in is read-only.
    # From the EXT_shader_framebuffer_fetch extension: "[...] fragment
    # shaders should use the read-only input array gl_LastFragData."
    #
    gen_compiler("""\
        /*
         * [config]
         * expect_result: fail
         * glsl_version: 1.0 es
         * require_extensions: GL_${extension}
         * [end config]
         */
        #version 100
        #extension GL_${extension} : enable

        void main()
        {
            gl_LastFragData[0] = vec4(1.0);
        }
    """, product(all_defs,
                 [{'name': 'negative-gl_LastFragData-write',
                   'api': 'gles2',
                   'shader_stage': 'frag'}]))

    #
    # Test framebuffer fetch output declarations with invalid layout
    # qualifiers.  From the EXT_shader_framebuffer_fetch extension:
    #
    # "It is an error to declare an inout fragment output not
    #  qualified with layout(noncoherent) if the
    #  GL_EXT_shader_framebuffer_fetch extension hasn't been enabled."
    #
    # "The ability to use the inout and layout(noncoherent) qualifiers
    #  at global scope in a fragment shader are optional and can be
    #  enabled by
    #   #extension GL_EXT_shader_framebuffer_fetch_non_coherent : <behavior>"
    #
    gen_compiler("""\
        /*
         * [config]
         * expect_result: fail
         * glsl_version: ${3.0 if api_version >= 3.0 else 1.0} es
         * require_extensions: GL_${extension}
         * [end config]
         */
        #version ${'300 es' if api_version >= 3.0 else '100'}
        #extension GL_${extension} : enable

        ${decl_frag_data(api_version, '' if layout else 'layout(noncoherent)')}

        void main()
        {
            ${last_frag_data(api_version)};
        }
    """, product(all_defs,
                 [{'name': 'negative-output-layout',
                   'shader_stage': 'frag'}],
                 [{'api': 'gles2', 'api_version': 2.0},
                  {'api': 'gles3', 'api_version': 3.0}]))

    #
    # Test that GL(ES) 3+ user-defined inout arrays are not accepted
    # in earlier GLSL versions.
    #
    gen_compiler("""\
         /*
          * [config]
          * expect_result: fail
          * glsl_version: 1.0 es
          * require_extensions: GL_${extension}
          * [end config]
          */
        #version 100
        #extension GL_${extension} : enable

        inout highp vec4 color;

        void main()
        {
            color += vec4(0.5);
        }
    """, product(all_defs,
                 [{'name': 'negative-inout-fragment-output',
                   'api': 'gles2',
                   'shader_stage': 'frag'}]))

    #
    # Test that redeclaring an existing built-in fragment output as
    # 'inout' leads to an error.
    #
    gen_compiler("""\
         /*
          * [config]
          * expect_result: fail
          * glsl_version: 3.0 es
          * require_extensions: GL_${extension}
          * [end config]
          */
        #version 300 es
        #extension GL_${extension} : enable

        inout highp float gl_FragDepth;

        void main()
        {
            gl_FragDepth += 0.5;
        }
    """, product(all_defs,
                 [{'name': 'negative-inout-gl_FragDepth',
                   'api': 'gles3',
                   'shader_stage': 'frag'}]))

    #
    # Test that vertex shader interface variables cannot be declared
    # 'inout'.
    #
    gen_compiler("""\
         /*
          * [config]
          * expect_result: fail
          * glsl_version: 3.0 es
          * require_extensions: GL_${extension}
          * [end config]
          */
        #version 300 es
        #extension GL_${extension} : enable

        inout highp vec4 vcolor;

        void main()
        {
        }
    """, product(all_defs,
                 [{'name': 'negative-inout-vertex-output',
                   'api': 'gles3',
                   'shader_stage': 'vert'}]))

    #
    # Test basic framebuffer fetch functionality.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= ${3.0 if api_version >= 3.0 else 1.0}
        %if samples > 0:
        INT GL_MAX_SAMPLES >= ${samples}
        %endif
        GL_${extension}

        [vertex shader passthrough]

        [fragment shader]
        #version ${'300 es' if api_version >= 3.0 else '100'}
        #extension GL_${extension} : enable

        ${decl_frag_data(api_version, layout)}

        void main()
        {
           ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                       vec4(0.5, 0.0, 0.0, 0.0);
        }

        [test]
        ${bind_fb('GL_RGBA8', samples)}

        clear color 0.0 0.0 1.0 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        ${resolve_fb('GL_RGBA8', samples)}

        relative probe rect rgb (0, 0.0, 1.0, 1.0) (1.0, 0.0, 1.0)

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'simple-'}],
                 [{'name': 'ss', 'api': 'gles2', 'api_version': 2.0, 'samples': 0},
                  {'name': 'ss', 'api': 'gles3', 'api_version': 3.0, 'samples': 0},
                  {'name': 'ms2', 'api': 'gles3', 'api_version': 3.0, 'samples': 2},
                  {'name': 'ms8', 'api': 'gles3', 'api_version': 3.0, 'samples': 8},
                  {'name': 'ms16', 'api': 'gles3', 'api_version': 3.0, 'samples': 16}]))

    #
    # Test read-back from a framebuffer with non-uniform contents
    # rendered during a previous draw call.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= ${3.0 if api_version >= 3.0 else 1.0}
        %if samples > 0:
        INT GL_MAX_SAMPLES >= ${samples}
        %endif
        GL_${extension}

        [vertex shader]
        #version ${'300 es' if api_version >= 3.0 else '100'}

        ${'in' if api_version >= 3.0 else 'attribute'} highp vec4 vertex;
        ${'out' if api_version >= 3.0 else 'varying'} highp vec4 vcolor;

        void main()
        {
            gl_Position = vertex;
            // Transform the vertex coordinates so that the R and G
            // components of the vertex color range between 0.0 and 1.0.
            vcolor = vec4((1.0 + vertex.x) / 2.0,
                          (1.0 + vertex.y) / 2.0, 0.0, 1.0);
        }

        [fragment shader]
        #version ${'300 es' if api_version >= 3.0 else '100'}
        #extension GL_${extension} : enable

        ${'in' if api_version >= 3.0 else 'varying'} highp vec4 vcolor;
        ${decl_frag_data(api_version, layout, p=precision)}

        void main()
        {
            // The condition makes sure that the else branch is
            // taken for the top and right quadrants during the second
            // overdraw.
            if (${last_frag_data(api_version)}.x <= 0.5 &&
                ${last_frag_data(api_version)}.y <= 0.5) {
                ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                            vcolor;
            } else {
                // Will give a solid color as result different for
                // each quadrant, assuming that the first branch was
                // taken during the first pass.
                ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                            vec4((vcolor.x >= 0.5 ? 1.0 : 0.0) - vcolor.x,
                                                 (vcolor.y >= 0.5 ? 1.0 : 0.0) - vcolor.y,
                                                 0, 0);
            }
        }

        [test]
        ${bind_fb('GL_RGBA8', samples)}

        clear color 0.0 0.0 1.0 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        ${resolve_fb('GL_RGBA8', samples)}

        relative probe rect rgb (0.55, 0.0, 0.45, 0.45) (1.0, 0.0, 1.0)
        relative probe rect rgb (0.0, 0.55, 0.45, 0.45) (0.0, 1.0, 1.0)
        relative probe rect rgb (0.55, 0.55, 0.45, 0.45) (1.0, 1.0, 1.0)

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'nonuniform-'}],
                 [{'name': 'ss', 'api': 'gles2', 'api_version': 2.0,
                   'samples': 0, 'precision': 'mediump'},
                   {'name': 'ss-redecl-highp', 'api': 'gles2',
                    'api_version': 2.0, 'samples': 0, 'precision': 'highp'},
                   {'name': 'ss-redecl-lowp', 'api': 'gles2',
                    'api_version': 2.0, 'samples': 0, 'precision': 'lowp'},
                   {'name': 'ss', 'api': 'gles3', 'api_version': 3.0,
                   'samples': 0, 'precision': 'mediump'},
                   {'name': 'ms2', 'api': 'gles3', 'api_version': 3.0,
                   'samples': 2, 'precision': 'mediump'},
                   {'name': 'ms8', 'api': 'gles3', 'api_version': 3.0,
                   'samples': 8, 'precision': 'mediump'},
                   {'name': 'ms16', 'api': 'gles3', 'api_version': 3.0,
                   'samples': 16, 'precision': 'mediump'}]))

    #
    # Test basic framebuffer fetch functionality in combination with
    # texturing.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= 3.00
        GL_${extension}

        [vertex shader passthrough]

        [fragment shader]
        #version 300 es
        #extension GL_${extension} : enable

        uniform sampler2D s;
        ${decl_frag_data(api_version, layout)}

        void main()
        {
            ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                        texelFetch(s, ivec2(gl_FragCoord), 0) / 4.0;
        }

        [test]
        ${bind_fb('GL_RGBA8')}

        texture rgbw 1 (250, 250) GL_RGBA8
        uniform int s 1

        clear color 0.5 0.0 0.0 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        relative probe rect rgb (0.0, 0.0, 0.45, 0.45) (1.0, 0.0, 0.0)
        relative probe rect rgb (0.55, 0.0, 0.45, 0.45) (0.5, 0.5, 0.0)
        relative probe rect rgb (0.0, 0.55, 0.45, 0.45) (0.5, 0.0, 0.5)
        relative probe rect rgb (0.55, 0.55, 0.45, 0.45) (1.0, 0.5, 0.5)

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'texture',
                   'api': 'gles3',
                   'api_version': 3.0}]))

    #
    # Test non-uniform fragment discard dependent on the result read
    # back from the framebuffer.  The EXT_shader_framebuffer_fetch
    # multisample test will be skipped if the implementation doesn't
    # claim to support per-sample discard.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= 3.00
        GL_${extension}
        %if samples > 0:
        INT GL_MAX_SAMPLES >= ${samples}
        %endif
        INT GL_FRAGMENT_SHADER_DISCARDS_SAMPLES_EXT >= ${1 if samples else 0}

        [vertex shader]
        #version 300 es

        in highp vec4 vertex;
        out highp vec4 vcolor;

        void main()
        {
            gl_Position = vertex;
            // Transform the vertex coordinates so that the R and G
            // components of the vertex color range between 0.0 and 1.0.
            vcolor = vec4((1.0 + vertex.x) / 2.0,
                          (1.0 + vertex.y) / 2.0, 0.0, 1.0);
        }

        [fragment shader]
        #version 300 es
        #extension GL_${extension} : enable

        in highp vec4 vcolor;
        ${decl_frag_data(api_version, layout)}

        void main()
        {
            // The condition makes sure that the discard branch is
            // taken for the top and right quadrants during the second
            // overdraw.
            if (${last_frag_data(api_version)}.x <= 0.45 &&
                ${last_frag_data(api_version)}.y < 0.45)
                ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                            vec4(vcolor.x >= 0.5 ? 0.5 : 0.1,
                                                 vcolor.y >= 0.5 ? 0.5 : 0.1,
                                                 0.0, 0.0);
            else
                discard;
        }

        [test]
        ${bind_fb('GL_RGBA8', samples)}

        clear color 0.0 0.0 1.0 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        ${resolve_fb('GL_RGBA8', samples)}

        relative probe rect rgb (0.0, 0.0, 0.45, 0.45) (0.2, 0.2, 1.0)
        relative probe rect rgb (0.55, 0.0, 0.45, 0.45) (0.5, 0.1, 1.0)
        relative probe rect rgb (0.0, 0.55, 0.45, 0.45) (0.1, 0.5, 1.0)
        relative probe rect rgb (0.55, 0.55, 0.45, 0.45) (0.5, 0.5, 1.0)

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'discard-',
                   'api': 'gles3',
                   'api_version': 3.0}],
                 [{'name': 'ss', 'samples': 0},
                  {'name': 'ms8', 'samples': 8}]))

    #
    # Test read-back from an integer framebuffer with non-uniform
    # contents rendered during a previous draw call.  Similar to the
    # "nonuniform" test above but using an integer framebuffer.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= 3.00
        GL_${extension}
        %if samples > 0:
        INT GL_MAX_SAMPLES >= ${samples}
        %endif

        [vertex shader]
        #version 300 es

        in highp vec4 vertex;
        out highp vec4 vcolor;

        void main()
        {
            gl_Position = vertex;
            // Transform the vertex coordinates so that the R and G
            // components of the vertex color range between 0.0 and 1.0.
            vcolor = vec4((1.0 + vertex.x) / 2.0,
                          (1.0 + vertex.y) / 2.0, 0.0, 1.0);
        }

        [fragment shader]
        #version 300 es
        #extension GL_${extension} : enable
        #define SCALE 100

        in highp vec4 vcolor;
        ${decl_frag_data(api_version, layout, t='ivec4')}

        void main()
        {
           if (${last_frag_data(api_version)}.x <= SCALE / 2 &&
               ${last_frag_data(api_version)}.y <= SCALE / 2)
              ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                          ivec4(vcolor * float(SCALE));
           else
              ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                          ivec4((vcolor.x >= 0.5 ? SCALE : 0)
                                                 - int(vcolor.x * float(SCALE)),
                                                (vcolor.y >= 0.5 ? SCALE : 0)
                                                 - int(vcolor.y * float(SCALE)),
                                                0, 0);
        }

        [test]
        ${bind_fb('GL_RGBA8I', samples)}

        clear color 0 0 1 0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        ${resolve_fb('GL_RGBA8I', samples)}

        relative probe rect rgba int (0.55, 0.0, 0.45, 0.45) (100, 0, 127, 100)
        relative probe rect rgba int (0.0, 0.55, 0.45, 0.45) (0, 100, 127, 100)
        relative probe rect rgba int (0.55, 0.55, 0.45, 0.45) (100, 100, 127, 100)
    """, product(all_defs,
                 [{'name': 'integer-', 'api': 'gles3'}],
                 [{'name': 'ss', 'samples': 0, 'api_version': 3.0},
                  {'name': 'ms2', 'samples': 2, 'api_version': 3.1},
                  {'name': 'ms8', 'samples': 8, 'api_version': 3.1}]))

    #
    # Test framebuffer fetch functionality in combination with
    # multiple render targets.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= ${3.0 if api_version >= 3.0 else 1.0}
        GL_${extension}

        [vertex shader passthrough]

        [fragment shader]
        #version ${'300 es' if api_version >= 3.0 else '100'}
        #extension GL_${extension} : enable

        ${decl_frag_data(api_version, layout, 4)}

        void main()
        {
           ${frag_data(api_version, 0)} = ${last_frag_data(api_version, 0)} +
                                          vec4(0.5, 0.0, 0.0, 0.0);
           ${frag_data(api_version, 1)} = ${last_frag_data(api_version, 1)} +
                                          vec4(0.0, 0.5, 0.0, 0.0);
           ${frag_data(api_version, 2)} = ${last_frag_data(api_version, 2)} +
                                          vec4(0.5, 0.5, 0.0, 0.0);
           ${frag_data(api_version, 3)} = ${last_frag_data(api_version, 3)} +
                                          vec4(0.0, 0.0, 0.5, 0.0);
        }

        [test]
        texture rgbw 0 (250, 250) GL_RGBA8
        texture rgbw 1 (250, 250) GL_RGBA8
        texture rgbw 2 (250, 250) GL_RGBA8
        texture rgbw 3 (250, 250) GL_RGBA8
        fb tex 2d 0 1 2 3

        clear color 0.0 0.0 0.5 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        fb tex 2d 0
        relative probe rect rgb (0.0, 0.0, 1.0, 1.0) (1.0, 0.0, 0.5)

        fb tex 2d 1
        relative probe rect rgb (0.0, 0.0, 1.0, 1.0) (0.0, 1.0, 0.5)

        fb tex 2d 2
        relative probe rect rgb (0.0, 0.0, 1.0, 1.0) (1.0, 1.0, 0.5)

        fb tex 2d 3
        relative probe rect rgb (0.0, 0.0, 1.0, 1.0) (0.0, 0.0, 1.0)

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'mrt'}],
                 [{'api': 'gles2', 'api_version': 2.0},
                  {'api': 'gles3', 'api_version': 3.0}]))

    #
    # Test framebuffer fetch functionality with multiple assignments
    # of the fragment color input-output.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= 3.00
        GL_${extension}

        [vertex shader]
        #version 300 es

        in vec4 vertex;
        out vec4 vcolor;

        void main()
        {
            gl_Position = vertex;
            // Transform the vertex coordinates so that the R and G
            // components of the vertex color range between 0.0 and 1.0.
            vcolor = vec4((1.0 + vertex.x) / 2.0,
                          (1.0 + vertex.y) / 2.0, 0.0, 1.0);
        }

        [fragment shader]
        #version 300 es
        #extension GL_${extension} : enable

        in highp vec4 vcolor;
        ${decl_frag_data(api_version, layout)}

        void main()
        {
            // The conditional assignment will be executed for the top
            // and right quadrants.
            if (vcolor.x >= 0.5 || vcolor.y >= 0.5)
                ${frag_data(api_version)} += vec4(0.5, 0, 0, 0);

            ${frag_data(api_version)} += vec4(0.0, 0.5, 0, 0);
        }

        [test]
        ${bind_fb('GL_RGBA8')}

        clear color 0.0 0.0 1.0 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        relative probe rect rgb (0.0, 0.0, 0.45, 0.45) (0.0, 1.0, 1.0)
        relative probe rect rgb (0.55, 0.0, 0.45, 0.45) (1.0, 1.0, 1.0)
        relative probe rect rgb (0.0, 0.55, 0.45, 0.45) (1.0, 1.0, 1.0)
        relative probe rect rgb (0.55, 0.55, 0.45, 0.45) (1.0, 1.0, 1.0)

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'overwrite',
                   'api': 'gles3',
                   'api_version': 3.0}]))

    #
    # Test framebuffer fetch functionality on individual slices of a
    # texture render target with multiple layers or mipmap levels.
    #
    gen_execution("""\
        [require]
        GL ES >= ${api_version}
        GLSL ES >= 3.00
        GL_${extension}

        [vertex shader passthrough]

        [fragment shader]
        #version 300 es
        #extension GL_${extension} : enable

        ${decl_frag_data(api_version, layout)}
        uniform highp vec4 ucolor;

        void main()
        {
            ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                        ucolor;
        }

        [test]
        %if target == 'Cube':
        texture storage 0 ${target} GL_RGBA8 (${levels} 250 250)
        %else:
        texture storage 0 ${target} GL_RGBA8 (${levels} 250 250 ${layers})
        %endif

        <%! blend_colors = ['0.5 0.0 0.0 0.0',
                            '0.0 0.5 0.0 0.0',
                            '0.5 0.5 0.0 0.0',
                            '0.0 0.0 0.5 0.0'] %>

        %for l in range(0, levels):
        %for z in range(0, layers):
        fb tex slice ${target} 0 ${l} ${z}
        uniform vec4 ucolor ${blend_colors[(l + z) % 4]}
        clear color 0.0 0.0 0.5 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        %endfor
        %endfor

        <%! expected_colors = ['(1.0, 0.0, 0.5)',
                               '(0.0, 1.0, 0.5)',
                               '(1.0, 1.0, 0.5)',
                               '(0.0, 0.0, 1.0)'] %>

        %for l in range(0, levels):
        %for z in range(0, layers):
        fb tex slice ${target} 0 ${l} ${z}
        relative probe rect rgb (0.0, 0.0, 1.0, 1.0) ${expected_colors[(l + z) % 4]}

        %endfor
        %endfor

        ${display_fb(api_version)}
    """, product(all_defs,
                 [{'name': 'single-slice-',
                   'api': 'gles3',
                   'api_version': 3.0}],
                 [{'name': '2darray', 'target': '2DArray',
                   'levels': 1, 'layers': 4},
                  {'name': '2darray-mipmap', 'target': '2DArray',
                   'levels': 4, 'layers': 1},
                  {'name': '3d', 'target': '3D',
                   'levels': 1, 'layers': 4},
                  {'name': 'cubemap', 'target': 'Cube',
                   'levels': 1, 'layers': 6}]))

    #
    # Test framebuffer fetch functionality on a 1D framebuffer.
    #
    gen_execution("""\
        [require]
        GL >= ${api_version}
        GLSL >= 1.50
        GL_ARB_texture_storage
        GL_${extension}

        [vertex shader]
        #version 150

        in vec4 vertex;
        out vec4 vcolor;

        void main()
        {
            gl_Position = vertex;
            // Transform the vertex coordinates so that the R
            // component of the vertex color ranges between 0.0 and 1.0.
            vcolor = vec4((1.0 + vertex.x) / 2.0, 0.7, 0.0, 1.0);
        }

        [fragment shader]
        #version 150
        #extension GL_${extension} : enable

        in vec4 vcolor;
        ${decl_frag_data(api_version, layout)}

        void main()
        {
            // The else branch will be executed during the second
            // overdraw.
            if (${last_frag_data(api_version)}.x <= 0.5 &&
                ${last_frag_data(api_version)}.y <= 0.5) {
                ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                            vcolor;
            } else {
                // Will give a solid color as result different for
                // each half, assuming that the first branch was taken
                // during the first pass.
                ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                                            vec4((vcolor.x >= 0.5 ? 1.0 : 0.0) - vcolor.x,
                                                 (vcolor.y >= 0.5 ? 1.0 : 0.0) - vcolor.y, 0, 0);
            }
        }

        [test]
        texture storage 0 1D GL_RGBA8 (1 250)
        fb tex slice 1D 0 0 0

        clear color 0.0 0.0 1.0 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        relative probe rect rgb (0.0, 0.0, 0.45, 1.0) (0.0, 1.0, 1.0)
        relative probe rect rgb (0.55, 0.0, 0.45, 1.0) (1.0, 1.0, 1.0)

        fb draw winsys
        blit color linear
    """, product(all_defs,
                 [{'name': '1d',
                   'api': 'gl',
                   'api_version': 3.2}]))

    #
    # Test framebuffer fetch functionality while rendering into
    # multiple layers of an array or cubemap framebuffer
    # simultaneously.
    #
    gen_execution("""\
        [require]
        GL >= ${api_version}
        GLSL >= 1.50
        GL_ARB_texture_storage
        GL_${extension}

        [vertex shader passthrough]
        [geometry shader]
        #version 150

        layout(triangles) in;
        layout(triangle_strip, max_vertices=18) out;

        flat out int glayer;

        void main()
        {
            for (int layer = 0; layer < ${layers}; layer++) {
                for (int i = 0; i < 3; i++) {
                    gl_Position = gl_in[i].gl_Position;
                    gl_Layer = glayer = layer;
                    EmitVertex();
                }
                EndPrimitive();
            }
        }

        [fragment shader]
        #version 150
        #extension GL_${extension} : enable

        flat in int glayer;
        ${decl_frag_data(api_version, layout)}

        void main()
        {
            ${frag_data(api_version)} = ${last_frag_data(api_version)} +
                (glayer == 5 ? vec4(0.0, 0.5, 0.5, 0.0) :
                 glayer == 4 ? vec4(0.5, 0.0, 0.5, 0.0) :
                 glayer == 3 ? vec4(0.0, 0.0, 0.5, 0.0) :
                 glayer == 2 ? vec4(0.5, 0.5, 0.0, 0.0) :
                 glayer == 1 ? vec4(0.0, 0.5, 0.0, 0.0) :
                 vec4(0.5, 0.0, 0.0, 0.0));
        }

        [test]
        texture storage 0 ${target} GL_RGBA8 (${dimensions})
        fb tex layered 0

        clear color 0.0 0.0 0.5 0.0
        clear
        ${barrier}
        draw rect -1 -1 2 2
        ${barrier}
        draw rect -1 -1 2 2

        <%! expected_colors = ['(1.0, 0.0, 0.5)',
                               '(0.0, 1.0, 0.5)',
                               '(1.0, 1.0, 0.5)',
                               '(0.0, 0.0, 1.0)',
                               '(1.0, 0.0, 1.0)',
                               '(0.0, 1.0, 1.0)'] %>

        %for z in range(0, layers):
        fb tex slice ${target} 0 0 ${z}
        relative probe rect rgb (0.0, 0.0, 1.0, 1.0) ${expected_colors[z]}
        %endfor

        fb draw winsys
        blit color linear
    """, product(all_defs,
                 [{'name': 'layered-',
                   'api': 'gl',
                   'api_version': 3.2}],
                 [{'name': '1darray', 'target': '1DArray',
                   'dimensions': '1 250 4', 'layers': 4},
                  {'name': '2darray', 'target': '2DArray',
                   'dimensions': '1 250 250 4', 'layers': 4},
                  {'name': 'cubemap', 'target': 'Cube',
                   'dimensions': '1 250 250', 'layers': 6}]))

if __name__ == '__main__':
    main()



# coding=utf-8
# Copyright (c) 2014, 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Tests for the exectest module """

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import os
import textwrap
try:
    import subprocess32 as subprocess
except ImportError:
    import subprocess

import pytest
import six

from six.moves import range

from framework import dmesg
from framework import log
from framework import monitoring
from framework import status
from framework.options import _Options as Options
from framework.test import base

from ..test_status import PROBLEMS
from .. import skip

# pylint: disable=invalid-name,no-self-use,protected-access


class _Test(base.Test):
    """Helper with stubbed interpret_results method."""
    def interpret_result(self):
        super(_Test, self).interpret_result()


class TestTest(object):
    """Tests for the Test class."""

    class TestRun(object):
        """Tests for Test.run."""

        def test_return_early_when_run_command_excepts(self, mocker):
            """Test.run exits early when Test._run_command raises an exception.
            """
            t = _Test(['foo'])
            mocker.patch.object(t, '_run_command',
                                side_effect=base.TestRunError('foo', 'pass'))
            mocker.patch.object(t, 'interpret_result',
                                side_effect=Exception('Test failure'))

            t.run()

    @pytest.mark.skipif(six.PY2 and subprocess.__name__ != 'subprocess32',
                        reason='Python 2.7 requires subprocess32 to run this test')
    @skip.posix
    class TestRunCommand(object):
        """Tests for Test._run_command."""

        @classmethod
        def setup_class(cls):
            if os.name == 'posix':
                cls.command = ['sleep', '60']
            else:
                cls.command = ['timeout', '/t', '/nobreak', '60']

        @pytest.mark.timeout(6)
        def test_timeout_kill_children(self, mocker, tmpdir):
            """test.base.Test: kill children if terminate fails.

            This creates a process that forks multiple times, and then checks
            that the children have been killed.

            This test could leave processes running if it fails.
            """
            # This function is the only user of psutil, and by putting it in
            # the function itself we avoid needed to try/except it, and then
            # skip if we don't have it.
            import psutil

            class PopenProxy(object):
                """An object that proxies Popen, and saves the Popen instance
                as an attribute.

                This is useful for testing the Popen instance.
                """

                def __init__(self):
                    self.popen = None

                def __call__(self, *args, **kwargs):
                    self.popen = subprocess.Popen(*args, **kwargs)

                    # if communicate is called successfully then the proc will
                    # be reset to None, which will make the test fail.
                    self.popen.communicate = mocker.Mock(
                        return_value=('out', 'err'))

                    return self.popen

            # localpath doesn't have a seek option
            with tmpdir.join('test').open(mode='w', ensure=True) as f:
                # Create a file that wll be executed as a python script Create
                # a process with two subproccesses (not threads) that will run
                # for a long time.
                f.write(textwrap.dedent("""\
                    import time
                    from multiprocessing import Process

                    def p():
                        for _ in range(100):
                            time.sleep(1)

                    if __name__ == "__main__":
                        a = Process(target=p)
                        b = Process(target=p)
                        a.start()
                        b.start()
                        a.join()
                        b.join()
                """))
                f.seek(0)  # we'll need to read the file back

                # Create an object that will return a popen object, but also
                # store it so we can access it later
                proxy = PopenProxy()

                test = _Test(['python' + ('2' if six.PY2 else '3'),
                              six.text_type(f)])
                test.timeout = 1

                # mock out subprocess.Popen with our proxy object
                mock_subp = mocker.patch('framework.test.base.subprocess')
                mock_subp.Popen = proxy
                mock_subp.TimeoutExpired = subprocess.TimeoutExpired
                test.run()

                # Check to see if the Popen has children, even after it should
                # have received a TimeoutExpired.
                proc = psutil.Process(os.getsid(proxy.popen.pid))
                children = proc.children(recursive=True)

            if children:
                # If there are still running children attempt to clean them up,
                # starting with the final generation and working back to the
                # first
                for child in reversed(children):
                    child.kill()

                raise Exception('Test process had children when it should not')

        @pytest.mark.slow
        @pytest.mark.timeout(6)
        def test_timeout(self):
            """test.base.Test: Stops running test after timeout expires.

            This is a little bit of extra time here, but without a sleep of 60
            seconds if the test runs 6 seconds it's run too long, we do need to
            allow a bit of time for teardown to happen.
            """
            test = _Test(self.command)
            test.timeout = 1
            test.run()

        @pytest.mark.slow
        @pytest.mark.timeout(6)
        def test_timeout_status(self):
            """test.base.Test: Setst the status to 'timeout' when then timeout
            is exceeded

            This is a little bit of extra time here, but without a sleep of 60
            seconds if the test runs 6 seconds it's run too long, we do need to
            allow a bit of time for teardown to happen.
            """
            test = _Test(self.command)
            test.timeout = 1
            test.run()
            assert test.result.result is status.TIMEOUT

    class TestExecuteTraceback(object):
        """Test.execute tests for Traceback handling."""

        class Sentinal(Exception):
            pass

        @pytest.fixture
        def shared_test(self, mocker):
            """Do some setup."""
            test = _Test(['foo'])
            test.run = mocker.Mock(side_effect=self.Sentinal)

            test.execute(mocker.Mock(spec=six.text_type),
                         mocker.Mock(spec=log.BaseLog),
                         {'dmesg': mocker.Mock(spec=dmesg.BaseDmesg),
                          'monitor': mocker.Mock(spec=monitoring.Monitoring)})
            return test.result

        def test_result(self, shared_test):
            """Test.execute (exception): Sets the result to fail."""
            assert shared_test.result is status.FAIL

        def test_traceback(self, shared_test):
            """Test.execute (exception): Sets the traceback.

            It's fragile to record the actual traceback, and it's unlikely that
            it can easily be implemented differently than the way the original
            code is implemented, so this doesn't do that, it just verifies
            there is a value.
            """
            assert shared_test.traceback != ''
            assert isinstance(shared_test.traceback, six.string_types)

        def test_exception(self, shared_test):
            """Test.execute (exception): Sets the exception.

            This is much like the traceback, it's difficult to get the correct
            value, so just make sure it's being set.
            """
            assert shared_test.exception != ''
            assert isinstance(shared_test.exception, six.string_types)

    class TestCommand(object):
        """Tests for Test.command."""

        def test_string_for_command(self):
            """Asserts if it is passed a string instead of a list."""
            with pytest.raises(AssertionError):
                _Test('foo')

        def test_mutation(self):
            """test.base.Test.command: does not mutate the value it was
            provided.

            There was a very subtle bug in all.py that causes the key values to
            be changed before they are assigned in some cases. This is because
            the right side of an assignment is evalated before the left side,
            so:

            >>> profile = {}
            >>> args = ['a', 'b']
            >>> profile[' '.join(args)] = PiglitGLTest(args)
            >>> list(profile.keys())
            ['bin/a b']
            """
            class MyTest(_Test):
                def __init__(self, *args, **kwargs):
                    super(MyTest, self).__init__(*args, **kwargs)
                    self._command[0] = 'bin/' + self._command[0]

            args = ['a', 'b']
            _Test(args)

            assert args == ['a', 'b']

    class TestInterpretResult(object):
        """Tests for Test.interpret_result."""

        def test_returncode_greater_zero(self):
            """A test with status > 0 is fail."""
            test = _Test(['foobar'])
            test.result.returncode = 1
            test.result.out = 'this is some\nstdout'
            test.result.err = 'this is some\nerrors'
            test.interpret_result()

            assert test.result.result is status.FAIL

        def test_crash_subtest_before_start(self):
            """A test for a test with a subtest, that crashes at the start
            of the run.
            """
            test = _Test(['foobar'])
            test.result.returncode = -1
            for x in (str(y) for y in range(5)):
                test.result.subtests[x] = status.NOTRUN
            test.interpret_result()

            assert test.result.result is status.CRASH
            assert test.result.subtests['0'] is status.CRASH
            for x in (str(y) for y in range(1, 5)):
                assert test.result.subtests[x] is status.NOTRUN

        def test_crash_subtest_mid(self):
            """A test for a test with a subtest, that crashes in the middle
            of the run.
            """
            test = _Test(['foobar'])
            test.result.returncode = -1
            for x in (str(y) for y in range(2)):
                test.result.subtests[x] = status.PASS
            for x in (str(y) for y in range(2, 5)):
                test.result.subtests[x] = status.NOTRUN
            test.interpret_result()

            assert test.result.result is status.CRASH
            for x in (str(y) for y in range(2)):
                assert test.result.subtests[x] is status.PASS
            assert test.result.subtests['2'] is status.CRASH
            for x in (str(y) for y in range(3, 5)):
                assert test.result.subtests[x] is status.NOTRUN


class TestWindowResizeMixin(object):
    """Tests for the WindowResizeMixin class."""

    def test_rerun(self):
        """test.base.WindowResizeMixin: runs multiple when spurious resize
        detected.
        """
        # Because of Python's inheritance order we need another mixin.
        class Mixin(object):
            def __init__(self, *args, **kwargs):
                super(Mixin, self).__init__(*args, **kwargs)
                self.__return_spurious = True

            def _run_command(self, *args, **kwargs):  # pylint: disable=unused-argument
                self.result.returncode = None

                # IF this is run only once we'll have "got spurious window resize"
                # in result.out, if it runs multiple times we'll get 'all good'
                if self.__return_spurious:
                    self.result.out = "Got spurious window resize"
                    self.__return_spurious = False
                else:
                    self.result.out = 'all good'

        class Test_(base.WindowResizeMixin, Mixin, _Test):
            pass

        test = Test_(['foo'])
        test.run()
        assert test.result.out == 'all good'


class TestValgrindMixin(object):
    """Tests for the ValgrindMixin class."""

    def test_command(self, mocker):
        """test.base.ValgrindMixin.command: overrides self.command."""
        opts = mocker.patch('framework.test.base.OPTIONS',
                            new_callable=Options)

        class Test(base.ValgrindMixin, _Test):
            pass

        opts.valgrind = True

        test = Test(['foo'])
        assert test.command == ['valgrind', '--quiet', '--error-exitcode=1',
                                '--tool=memcheck', 'foo']

    class TestRun(object):
        """Tests for the run method."""

        @classmethod
        def setup_class(cls):
            class _NoRunTest(_Test):
                def run(self):
                    self.interpret_result()

            class Test(base.ValgrindMixin, _NoRunTest):
                pass

            cls.test = Test

        # The ids function here is a bit of a hack to work around the
        # pytest-timeout plugin, which is broken. when 'timeout' is passed as a
        # string using six.text_type it tries to grab that value and a flaming
        # pile ensues
        @pytest.mark.parametrize("starting", PROBLEMS,
                                 ids=lambda x: six.text_type(x).upper())
        def test_problem_status_changes_valgrind_enabled(self, starting, mocker):
            """When running with valgrind mode we're actually testing the test
            binary itself, so any status other than pass is irrelavent, and
            should be marked as skip.
            """
            mock_opts = mocker.patch('framework.test.base.OPTIONS',
                                     new_callable=Options)
            mock_opts.valgrind = True

            test = self.test(['foo'])
            test.result.result = starting
            test.run()
            assert test.result.result is status.SKIP

        @pytest.mark.parametrize("starting", PROBLEMS,
                                 ids=lambda x: six.text_type(x).upper())
        def test_problems_with_valgrind_disabled(self, starting, mocker):
            """When valgrind is disabled nothign shoud change
            """
            mock_opts = mocker.patch('framework.test.base.OPTIONS',
                                     new_callable=Options)
            mock_opts.valgrind = False

            test = self.test(['foo'])
            test.result.result = starting
            test.result.returncode = 0
            test.run()
            assert test.result.result is starting

        def test_passed_valgrind(self, mocker):
            """test.base.ValgrindMixin.run: when test is 'pass' and returncode
            is '0' result is pass.
            """
            mock_opts = mocker.patch('framework.test.base.OPTIONS',
                                     new_callable=Options)
            test = self.test(['foo'])
            mock_opts.valgrind = True
            test.result.result = status.PASS
            test.result.returncode = 0
            test.run()
            assert test.result.result is status.PASS

        def test_failed_valgrind(self, mocker):
            """test.base.ValgrindMixin.run: when a test is 'pass' but
            returncode is not 0 it's 'fail'.
            """
            mock_opts = mocker.patch('framework.test.base.OPTIONS',
                                     new_callable=Options)
            mock_opts.valgrind = True
            test = self.test(['foo'])
            test.result.result = status.PASS
            test.result.returncode = 1
            test.interpret_result()
            assert test.result.result is status.FAIL


class TestReducedProcessMixin(object):
    """Tests for the ReducedProcessMixin class."""

    class MPTest(base.ReducedProcessMixin, _Test):
        def _resume(self, current):
            return [self.command[0]] + self._expected[current:]

        def _is_subtest(self, line):
            return line.startswith('TEST')

    def test_populate_subtests(self):
        test = self.MPTest(['foobar'], subtests=['a', 'b', 'c'])
        assert set(test.result.subtests.keys()) == {'a', 'b', 'c'}

    class TestRunCommand(object):
        """Tests for the _run_command method."""

        @pytest.fixture(scope='module')
        def test_class(self):
            """Defines a test class that uses generators to ease testing."""
            class _Shim(object):
                """This shim goes between the Mixin and the Test class and
                provides a way to set the output of the test.
                """

                def __init__(self, *args, **kwargs):
                    super(_Shim, self).__init__(*args, **kwargs)
                    self.gen_rcode = None
                    self.gen_out = None
                    self.get_err = None

                def _run_command(self, *args, **kwargs):  # pylint: disable=unused-argument
                    # pylint: disable=no-member
                    self.result.returncode = next(self.gen_rcode)
                    self.result.out = next(self.gen_out)
                    self.result.err = next(self.gen_err)

            class Test(base.ReducedProcessMixin, _Shim, _Test):
                """The actual Class returned by the fixture.

                This class implements the abstract bits from
                ReducedProcessMixin, and inserts the _Shim class. The
                _is_subtest method is implemented such that any line starting
                with SUBTEST is a subtest.
                """

                def _is_subtest(self, line):
                    return line.startswith('SUBTEST')

                def _resume(self, cur, **kwargs):  # pylint: disable=unused-argument
                    return self._expected[cur:]

                def interpret_result(self):
                    name = None

                    for line in self.result.out.split('\n'):
                        if self._is_subtest(line):
                            name = line[len('SUBTEST: '):]
                        elif line.startswith('RESULT: '):
                            self.result.subtests[name] = line[len('RESULT: '):]
                            name = None

            return Test

        def test_result(self, test_class):
            """Test result attributes."""
            test = test_class(['foobar'], ['a', 'b'])
            test.gen_out = iter(['SUBTEST: a', 'SUBTEST: b'])
            test.gen_err = iter(['err output', 'err output'])
            test.gen_rcode = iter([2, 0])
            test._run_command()
            assert test.result.out == \
                'SUBTEST: a\n\n====RESUME====\n\nSUBTEST: b'
            assert test.result.err == \
                'err output\n\n====RESUME====\n\nerr output'
            assert test.result.returncode == 2

        @pytest.mark.timeout(5)
        def test_infinite_loop(self, test_class):
            """Test that we don't get into an infinite loop."""
            test = test_class(['foobar'], ['a', 'b'])
            test.gen_out = iter(['a', 'a'])
            test.gen_err = iter(['a', 'a'])
            test.gen_rcode = iter([1, 1])
            test._run_command()

        def test_crash_first(self, test_class):
            """Handles the first test crashing."""
            test = test_class(['foo'], ['a', 'b'])
            test.gen_out = iter(['', 'SUBTEST: a'])
            test.gen_err = iter(['', ''])
            test.gen_rcode = iter([1, 0])

            # Since interpret_result isn't called this would normally be left
            # as NOTRUN, but we want to ensure that _run_command isn't mucking
            # with it, so we set it to this PASS, which acts as a sentinal
            test.result.subtests['b'] = status.PASS
            test._run_command()

            assert test.result.subtests['a'] is status.CRASH
            assert test.result.subtests['b'] is status.PASS

        def test_middle_crash(self, test_class):
            """handle the final subtest crashing."""
            test = test_class(['foo'], ['a', 'b', 'c'])
            test.gen_out = iter(['SUBTEST: a\nRESULT: pass\nSUBTEST: b\n',
                                 'SUBTEST: c\nRESULT: pass\n'])
            test.gen_err = iter(['', ''])
            test.gen_rcode = iter([1, 0])

            test._run_command()
            test.interpret_result()

            assert test.result.subtests['a'] == status.PASS
            assert test.result.subtests['b'] == status.CRASH
            assert test.result.subtests['c'] == status.PASS

        def test_final_crash(self, test_class):
            """handle the final subtest crashing."""
            test = test_class(['foo'], ['a', 'b', 'c'])
            test.gen_out = iter(['SUBTEST: a\nRESULT: pass\n'
                                 'SUBTEST: b\nRESULT: pass\n'
                                 'SUBTEST: c\n'])
            test.gen_err = iter([''])
            test.gen_rcode = iter([1])

            test._run_command()
            test.interpret_result()

            assert test.result.subtests['a'] == status.PASS
            assert test.result.subtests['b'] == status.PASS
            assert test.result.subtests['c'] == status.CRASH



# encoding=utf-8
# Copyright © 2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides helper classes for representing glsl data."""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import functools

import six

import compat

__all__ = [
    'GLSLESVersion',
    'GLSLVersion',
    'Version',
]


class _Version(object):
    """Factory class for glsl versions.

    provides an object cache to reduce duplication of objects. This object
    should not be initialized more than once, to avoid that use the Version
    constant provided by the module.

    It would provide either a GLSLVersion or a GLSLESVersion.

    """
    _es_versions = [
        '100',
        '300 es',
        '310 es',
        '320 es',
    ]

    _versions = [
        '110',
        '120',
        '130',
        '140',
        '150',
        '330',
        '400',
        '410',
        '420',
        '430',
        '440',
        '450',
    ]

    def __init__(self):
        self.__cache = {}

    def __call__(self, ver):
        """Make a Version object, or provide one from the cache."""
        assert isinstance(ver, six.text_type)

        # Try to get an object from the cache, if that fails create a new one
        # and add it to the cache before returning it.
        try:
            return self.__cache[ver]
        except KeyError:
            if ver in self._es_versions:
                obj = GLSLESVersion(ver)
            elif ver in self._versions:
                obj = GLSLVersion(ver)
            else:
                raise Exception('Undefined version {}'.format(ver))

            self.__cache[ver] = obj
            return obj


Version = _Version()  # pylint: disable=invalid-name


@compat.python_2_unicode_compatible  # pylint: disable=no-member
@functools.total_ordering
class GLSLVersion(object):
    """A Representation of an OpenGL Shading Language version.

    This object provides a bunch of the niceties that one would want. It's
    orderable (can be sorted, and can be compared with the standard ==, !=, <,
    etc), can be called with str() (which provides the integer version, like
    120), and a method to print the decimal version (1.20).

    Do not initialize this directly.

    """
    def __init__(self, ver):
        assert ver in ['110', '120', '130', '140', '150', '330', '400', '410',
                       '420', '430', '440', '450']
        self._internal = ver
        self.is_es = False

    def __str__(self):
        return self._internal

    def __int__(self):
        return int(self._internal)

    def __float__(self):
        return float(int(self) / 100)

    def __eq__(self, other):
        # If the other version is ES then we know they're not equal
        if isinstance(other, GLSLESVersion):
            return False
        elif isinstance(other, (GLSLVersion, int)):
            return int(self) == int(other)
        elif isinstance(other, float):
            return float(self) == float(other)
        else:
            return NotImplemented

    def __ne__(self, other):
        return not self == other

    def __lt__(self, other):
        if isinstance(other, GLSLESVersion):
            raise TypeError('Unorderable types GLSLVersion and GLSLESVersion')
        elif isinstance(other, (GLSLVersion, int)):
            return int(self) < int(other)
        elif isinstance(other, float):
            return float(self) < other
        else:
            return NotImplemented

    def print_float(self):
        """A version suitable to print as a decimal with two trailing digits."""
        return '{:.2f}'.format(float(self))


@compat.python_2_unicode_compatible  # pylint: disable=no-member
@functools.total_ordering
class GLSLESVersion(object):
    """Represents a GLSL ES version.

    Do not initialize this directly.

    """
    def __init__(self, ver):
        if ver == '100':
            self._internal = ver
        else:
            self._internal = ver[:-3]  # drop " es"
        self.is_es = True

    def __str__(self):
        if self._internal == '100':
            return self._internal
        else:
            return self._internal + " es"

    def __int__(self):
        return int(self._internal)

    def __float__(self):
        return float(int(self) / 100)

    def __eq__(self, other):
        # If the other version is ES then we know they're not equal
        if isinstance(other, GLSLVersion):
            return False
        elif isinstance(other, (GLSLESVersion, int)):
            return int(self) == int(other)
        elif isinstance(other, float):
            return float(self) == float(other)
        else:
            return NotImplemented

    def __ne__(self, other):
        return not self == other

    def __lt__(self, other):
        if isinstance(other, GLSLVersion):
            raise TypeError('Unorderable types GLSLESVersion and GLSLVersion')
        elif isinstance(other, (GLSLESVersion, int)):
            return int(self) < int(other)
        elif isinstance(other, float):
            return float(self) < other
        else:
            return NotImplemented

    def print_float(self):
        """A version suitable to print as a decimal with two trailing digits."""
        if self._internal == '100':
            return '{:.2f}'.format(float(self))
        else:
            return '{:.2f} es'.format(float(self))


class _MinVersion(object):
    """A factory class for sorting GLSL and GLSLES versions.

    This stores the minimum version required for various operations (currently
    only for_stage and for_stage_with_ext).

    This class is not meant to be reinitialized, instead use the provided
    MinVersion constant.

    """
    __gl_stage_min = {
        'frag': Version('110'),
        'vert': Version('110'),
        'geom': Version('150'),
        'tesc': Version('400'),
        'tese': Version('400'),
        'comp': Version('430'),
    }
    # Only versions that actaly change are here, the function will return the
    # values from __gl_stage_min if they're not here
    __gl_stage_min_ext = {
        # geometry_shader4 is not included here intentionally. It is
        # significantly different than the geometry shaders in OpenGL 3.2
        'tesc': (Version('140'), 'GL_ARB_tessellation_shader'),
        'tese': (Version('140'), 'GL_ARB_tessellation_shader'),
        'comp': (Version('140'), 'GL_ARB_compute_shader'),
    }
    __gles_stage_min = {
        'frag': Version('100'),
        'vert': Version('100'),
        'comp': Version('310 es'),
        'geom': Version('320 es'),
        'tesc': Version('320 es'),
        'tese': Version('320 es'),
    }
    # Only versions that actaly change are here, the function will return the
    # values from __gles_stage_min if they're not here
    __gles_stage_min_ext = {
        'geom': (Version('310 es'), 'GL_OES_geometry_shader'),
        'tesc': (Version('310 es'), 'GL_OES_tessellation_shader'),
        'tese': (Version('310 es'), 'GL_OES_tessellation_shader'),
    }

    def for_stage(self, stage, version):
        """Return max(stage minimum version, requested version).

        When provided a stage and a version, it will return the greater of the
        provided version and the minimum version of that stage without an
        extension. For example, in OpenGL teselation is available in GLSL
        4.00+, or in 1.40+ with ARB_tessellation_shader. Given Version('150')
        and 'tesc' this method returns Version('400').

        Arguments:
        stage -- A stage named by the extensions glslparsertest uses (frag,
                 vert, geom, tesc, tese, comp)
        version -- A version as returned by the Version function.

        >>> m = _MinVersion()
        >>> m.for_stage('geom', Version('300 es'))
        Version('320 es')
        >>> m.for_stage('frag', Version('130'))
        Version('130')

        """
        assert isinstance(version, (GLSLVersion, GLSLESVersion))
        if isinstance(version, GLSLVersion):
            _stage = self.__gl_stage_min[stage]
        elif isinstance(version, GLSLESVersion):
            _stage = self.__gles_stage_min[stage]

        return _stage if _stage > version else version

    def for_stage_with_ext(self, stage, version):
        """Return the earliest GLSL version that a stage is supported in with
        an extension.

        When provided a stage and a version, it will return the greater of the
        provided version and the minimum version of that stage with an
        extension, and if necissary the extension as a string. For example, in
        OpenGL teselation is available in GLSL 4.00+, or in 1.40+ with
        ARB_tessellation_shader. Given Version('150') and 'tesc' this method
        returns (Version('150'), 'GL_ARB_tessellation_shader'); but given
        Version('400') and 'tesc' it returns (Version('400'), None)

        If there is no extension (like with fragment and vertex) then None will
        be returned as the secon value. It is up to the caller to handle this
        appropriately. It will also return None for the extension when the GLSL
        version is high enough to not require an extension.

        Takes the same arguments as for_stage.

        >>> m = _MinVersion()
        >>> m.for_stage_with_ext('geom', Version('300 es'))
        (Version('310 es'), 'GL_OES_geometry_shader')
        >>> m.for_stage_with_ext('frag', Version('130'))
        (Version('130'), None)

        """
        assert isinstance(version, (GLSLVersion, GLSLESVersion))
        if isinstance(version, GLSLVersion):
            try:
                _stage, ext = self.__gl_stage_min_ext[stage]
            except KeyError:
                _stage, ext = self.__gl_stage_min[stage], None
        elif isinstance(version, GLSLESVersion):
            try:
                _stage, ext = self.__gles_stage_min_ext[stage]
            except KeyError:
                _stage, ext = self.__gles_stage_min[stage], None

        # If the version queried is less than the required, return the require
        # and the ext
        if _stage > version:
            return (_stage, ext)
        # If the requested version is greater or equal to the version that the
        # feature became core in, return the version and None
        elif self.for_stage(stage, version) <= version:
            return (version, None)
        # Otherwise the requested version and the extension are returned
        else:
            return (version, ext)


MinVersion = _MinVersion()  # pylint: disable=invalid-name



# coding=utf-8
# Copyright (c) 2015-2016 Intel Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Compression support for backends.

This includes both compression and decompression support.

This provides a low level interface of dictionaries, COMPRESSORS and
DECOMPRESSORS, which use compression modes ('bz2', 'gz', 'xz', 'none') to
provide open-like functions with correct mode settings for writing or reading,
respectively.

They should always take unicode (str in python 3.x) objects. It is up to the
caller to ensure that they're passing unicode and not bytes.

A helper, get_mode(), is provided to return the user selected mode (it will try
the PIGLIT_COMPRESSION environment variable, then the piglit.conf
[core]:compression key, and finally the value of compression.DEFAULT). This is
the best way to get a compressor.

"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)
import bz2
import errno
import functools
import gzip
import os
import subprocess
import contextlib

import six
from six.moves import cStringIO as StringIO

from framework import exceptions, compat
from framework.core import PIGLIT_CONFIG

__all__ = [
    'UnsupportedCompressor',
    'COMPRESSORS',
    'DECOMPRESSORS',
    'get_mode',
]


@compat.python_2_unicode_compatible
class UnsupportedCompressor(exceptions.PiglitInternalError):
    def __init__(self, method, *args, **kwargs):
        super(UnsupportedCompressor, self).__init__(*args, **kwargs)
        self.__method = method

    def __str__(self):
        return u'unsupported compression method {}'.format(self.__method)



DEFAULT = 'bz2'

if six.PY2:
    COMPRESSION_SUFFIXES = ['.gz', '.bz2']
    COMPRESSORS = {
        'bz2': functools.partial(bz2.BZ2File, mode='w'),
        'gz': functools.partial(gzip.open, mode='w'),
        'none': functools.partial(open, mode='w'),
    }

    DECOMPRESSORS = {
        'bz2': functools.partial(bz2.BZ2File, mode='r'),
        'gz': functools.partial(gzip.open, mode='r'),
        'none': functools.partial(open, mode='r'),
    }

    # First try to use backports.lzma, that's the easiest solution. If that
    # fails then go to trying the shell. If that fails then piglit won't have
    # xz support, and will raise an error if xz is used
    try:
        import backports.lzma  # pylint: disable=wrong-import-position

        COMPRESSORS['xz'] = functools.partial(backports.lzma.open, mode='w')
        DECOMPRESSORS['xz'] = functools.partial(backports.lzma.open, mode='r')
        COMPRESSION_SUFFIXES += ['.xz']
    except ImportError:
        try:
            with open(os.devnull, 'w') as d:
                subprocess.check_call(['xz', '--help'], stdout=d, stderr=d)
        except OSError:
            pass
        else:

            @contextlib.contextmanager
            def _compress_xz(filename):
                """Emulates an open function in write mode for xz.

                Python 2.x doesn't support xz, but it's dang useful. This
                function calls out to the shell and tries to use xz from the
                environment to get xz compression.

                This obviously won't work without a working xz binary.

                This function tries to emulate the default values of the lzma
                module in python3 as much as possible

                """
                if filename.endswith('.xz'):
                    filename = filename[:-3]

                with open(filename, 'w') as f:
                    yield f

                try:
                    with open(os.devnull, 'w') as null:
                        subprocess.check_call(
                            ['xz', '--compress', '-9', '--force', filename],
                            stderr=null)
                except OSError as e:
                    if e.errno == errno.ENOENT:
                        raise exceptions.PiglitFatalError(
                            'No xz binary available')
                    raise

            @contextlib.contextmanager
            def _decompress_xz(filename):
                """Eumlates an option function in read mode for xz.

                See the comment in _compress_xz for more information.

                This function tries to emulate the lzma module as much as
                possible

                """
                if not filename.endswith('.xz'):
                    filename = '{}.xz'.format(filename)

                try:
                    with open(os.devnull, 'w') as null:
                        string = subprocess.check_output(
                            ['xz', '--decompress', '--stdout', filename],
                            stderr=null)
                except OSError as e:
                    if e.errno == errno.ENOENT:
                        raise exceptions.PiglitFatalError(
                            'No xz binary available')
                    raise

                # We need a file-like object, so the contents must be placed in
                # a StringIO object.
                io = StringIO()
                io.write(string)
                io.seek(0)

                yield io

                io.close()

            COMPRESSORS['xz'] = _compress_xz
            DECOMPRESSORS['xz'] = _decompress_xz
            COMPRESSION_SUFFIXES += ['.xz']
else:
    # In the case of python 3 this all just works, no monkeying around with
    # imports and fallbacks. just import the right modules and go

    import lzma  # pylint: disable=wrong-import-position,wrong-import-order

    COMPRESSION_SUFFIXES = ['.gz', '.bz2', '.xz']

    COMPRESSORS = {
        'bz2': functools.partial(bz2.open, mode='wt'),
        'gz': functools.partial(gzip.open, mode='wt'),
        'none': functools.partial(open, mode='w'),
        'xz': functools.partial(lzma.open, mode='wt'),
    }

    DECOMPRESSORS = {
        'bz2': functools.partial(bz2.open, mode='rt'),
        'gz': functools.partial(gzip.open, mode='rt'),
        'none': functools.partial(open, mode='r'),
        'xz': functools.partial(lzma.open, mode='rt'),
    }


def get_mode():
    """Return the key value of the correct compressor to use.

    Try the environment variable PIGLIT_COMPRESSION; then check the
    PIGLIT_CONFIG section 'core', option 'compression'; finally fall back to
    DEFAULT.

    This will raise an UnsupportedCompressionError if there isn't a compressor
    for that mode. It is the job of the caller to handle this exceptions

    """
    # This is provided as a function rather than a constant because as a
    # function it can honor changes to the PIGLIT_CONFIG instance, or the
    # PIGLIT_COMPRESSION environment variable.

    method = (os.environ.get('PIGLIT_COMPRESSION') or
              PIGLIT_CONFIG.safe_get('core', 'compression') or
              DEFAULT)

    if method not in COMPRESSORS:
        raise UnsupportedCompressor(method)

    return method
